/*********************************************************************
 *                  Flowcode CAL CAN File
 *
 * File: PIC_CAL_CAN.c
 *
 * (c) 2011 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 230911 | BR | Created
 * 011211 | BR | Converted to new dynamic defines mechanism
 */


#ifndef MX_ECAN
	#error "Internal CAN not supported on this device"
#else

#ifndef INT_CAN_DEFS
  #define INT_CAN_DEFS
  #define CAN_OP_MODE_BITS 	0xE0
#endif

#define MX_CAN_CHANNEL_X		CAL_APPEND(MX_CAN_CHANNEL_, MX_CAN_NUM)
#define MX_CAN_BAUD1_X			CAL_APPEND(MX_CAN_BAUD1_, MX_CAN_NUM)
#define MX_CAN_BAUD2_X			CAL_APPEND(MX_CAN_BAUD2_, MX_CAN_NUM)
#define MX_CAN_BAUD3_X			CAL_APPEND(MX_CAN_BAUD3_, MX_CAN_NUM)


//Function Prototypes
CALFUNCTION(void, FC_CAL_Internal_CAN_Init_, (void));
CALFUNCTION(void, FC_CAL_Internal_CAN_Uninit_, (void));
CALFUNCTION(void, FC_CAL_Internal_CAN_Send_Buffer_, (MX_UINT8 buffer));
CALFUNCTION(MX_UINT8, FC_CAL_Internal_CAN_CheckRX_, (MX_UINT8 buffer));
CALFUNCTION(void, FC_CAL_Internal_CAN_Set_TX_Data_, (MX_UINT8 buffer, MX_UINT8 data_cnt, MX_UINT8 d0, MX_UINT8 d1, MX_UINT8 d2, MX_UINT8 d3, MX_UINT8 d4, MX_UINT8 d5, MX_UINT8 d6, MX_UINT8 d7));
CALFUNCTION(void, FC_CAL_Internal_CAN_Set_TX_Std_ID_, (MX_UINT8 buffer, MX_UINT8 hi, MX_UINT8 lo));
CALFUNCTION(void, FC_CAL_Internal_CAN_Set_TX_Exd_ID_, (MX_UINT8 buffer, MX_UINT8 id3, MX_UINT8 id2, MX_UINT8 id1, MX_UINT8 id0));
CALFUNCTION(void, FC_CAL_Internal_CAN_Set_Std_RX_Filter_, (MX_UINT8 filter, MX_UINT8 hi, MX_UINT8 lo));
CALFUNCTION(void, FC_CAL_Internal_CAN_Set_Exd_RX_Filter_, (MX_UINT8 filter, MX_UINT8 id3, MX_UINT8 id2, MX_UINT8 id1, MX_UINT8 id0));
CALFUNCTION(void, FC_CAL_Internal_CAN_Set_Std_RX_Mask_, (MX_UINT8 filter, MX_UINT8 hi, MX_UINT8 lo));
CALFUNCTION(void, FC_CAL_Internal_CAN_Set_Exd_RX_Mask, (MX_UINT8 filter, MX_UINT8 id3, MX_UINT8 id2, MX_UINT8 id1, MX_UINT8 id0));


CALFUNCTION(void, FC_CAL_Internal_CAN_Init_, (void))
{
	#if (MX_CAN_CHANNEL_X == 1)
		//configure CAN I/O
		ciocon = 0x30;

		// In order to setup necessary config parameters of CAN module,
		// it must be in CONFIG mode.

		// Request desired mode.
		cancon = REQOP_CONFIG;
		// Wait till desired mode is set.
		while( (canstat & CAN_OP_MODE_BITS) != REQOP_CONFIG );

		brgcon3 = MX_CAN_BAUD3_X;
		brgcon2 = MX_CAN_BAUD2_X;
		brgcon1 = MX_CAN_BAUD1_X;

		rxf0sidh = RXF0SIDH_VAL;    // ID Filter 0 - Buffer 0
		rxf0sidl = RXF0SIDL_VAL;
		rxf0eidh = RXF0EID8_VAL;
		rxf0eidl = RXF0EID0_VAL;
		rxf1sidh = RXF1SIDH_VAL;    // ID Filter 1 - Buffer 0
		rxf1sidl = RXF1SIDL_VAL;
		rxf1eidh = RXF1EID8_VAL;
		rxf1eidl = RXF1EID0_VAL;
		rxf2sidh = RXF2SIDH_VAL;    // ID Filter 2 - Buffer 1
		rxf2sidl = RXF2SIDL_VAL;
		rxf2eidh = RXF2EID8_VAL;
		rxf2eidl = RXF2EID0_VAL;
		rxf3sidh = RXF3SIDH_VAL;    // ID Filter 3 - Buffer 1
		rxf3sidl = RXF3SIDL_VAL;
		rxf3eidh = RXF3EID8_VAL;
		rxf3eidl = RXF3EID0_VAL;
		rxf4sidh = RXF4SIDH_VAL;    // ID Filter 4 - Buffer 1
		rxf4sidl = RXF4SIDL_VAL;
		rxf4eidh = RXF4EID8_VAL;
		rxf4eidl = RXF4EID0_VAL;
		rxf5sidh = RXF5SIDH_VAL;    // ID Filter 5 - Buffer 1
		rxf5sidl = RXF5SIDL_VAL;
		rxf5eidh = RXF5EID8_VAL;
		rxf5eidl = RXF5EID0_VAL;
		rxm0sidh = RXM0SIDH_VAL;    // ID Mask 0   - Buffer 0
		rxm0sidl = RXM0SIDL_VAL;
		rxm0eidh = RXM0EID8_VAL;
		rxm0eidl = RXM0EID0_VAL;
		rxm1sidh = RXM1SIDH_VAL;    // ID Mask 1   - Buffer 1
		rxm1sidl = RXM1SIDL_VAL;
		rxm1eidh = RXM1EID8_VAL;
		rxm1eidl = RXM1EID0_VAL;

		cancon = cancon & 0xE0;
		//cancon = cancon | 0x0C;
		rxb0con = RXB0CTRL_VAL;    	// Receive Buffer 0 Control

		cancon = cancon & 0xE0;
		cancon = cancon | 0x0A;
		rxb0con = RXB1CTRL_VAL;     // Receive Buffer 1 Control

		cancon = cancon & 0xE0;
		cancon = cancon | 0x08;
		rxb0con = TXB0CTRL_VAL;     // Transmit Buffer 0 Control
		rxb0sidh = TXB0SIDH_VAL;    // Transmit ID 0
		rxb0sidl = TXB0SIDL_VAL;
		rxb0eidh = TXB0EID8_VAL;
		rxb0eidl = TXB0EID0_VAL;
		rxb0dlc = TXB0DLC_VAL;
		rxb0d0 = TXB0DB0_VAL;      // Transmit Data Bytes
		rxb0d1 = TXB0DB1_VAL;
		rxb0d2 = TXB0DB2_VAL;
		rxb0d3 = TXB0DB3_VAL;
		rxb0d4 = TXB0DB4_VAL;
		rxb0d5 = TXB0DB5_VAL;
		rxb0d6 = TXB0DB6_VAL;
		rxb0d7 = TXB0DB7_VAL;

		cancon = cancon & 0xE0;
		cancon = cancon | 0x06;
		rxb0con = TXB1CTRL_VAL;     // Transmit Buffer 1 Control
		rxb0sidh = TXB1SIDH_VAL;    // Transmit ID 1
		rxb0sidl = TXB1SIDL_VAL;
		rxb0eidh = TXB1EID8_VAL;
		rxb0eidl = TXB1EID0_VAL;
		rxb0dlc = TXB1DLC_VAL;
		rxb0d1 = TXB1DB0_VAL;      // Transmit Data Bytes
		rxb0d1 = TXB1DB1_VAL;
		rxb0d2 = TXB1DB2_VAL;
		rxb0d3 = TXB1DB3_VAL;
		rxb0d4 = TXB1DB4_VAL;
		rxb0d5 = TXB1DB5_VAL;
		rxb0d6 = TXB1DB6_VAL;
		rxb0d7 = TXB1DB7_VAL;

		cancon = cancon & 0xE0;
		cancon = cancon | 0x04;
		rxb0con = TXB2CTRL_VAL;     // Transmit Buffer 2 Control
		rxb0sidh = TXB2SIDH_VAL;    // Transmit ID 2
		rxb0sidl = TXB2SIDL_VAL;
		rxb0eidh = TXB2EID8_VAL;
		rxb0eidl = TXB2EID0_VAL;
		rxb0dlc = TXB2DLC_VAL;
		rxb0d0 = TXB2DB0_VAL;      // Transmit Data Bytes
		rxb0d1 = TXB2DB1_VAL;
		rxb0d2 = TXB2DB2_VAL;
		rxb0d3 = TXB2DB3_VAL;
		rxb0d4 = TXB2DB4_VAL;
		rxb0d5 = TXB2DB5_VAL;
		rxb0d6 = TXB2DB6_VAL;
		rxb0d7 = TXB2DB7_VAL;

		// Request desired mode.
		cancon = REQOP_NORMAL;

		// Wait till desired mode is set.
		while( (canstat & CAN_OP_MODE_BITS) != REQOP_NORMAL );
	#endif

	#if (MX_CAN_CHANNEL_X == 2)
   		#error "8-Bit PICs only ever have 1 CAN channel..... For now";
	#endif
}

CALFUNCTION(void, FC_CAL_Internal_CAN_Uninit_, (void))
{
	#if (MX_CAN_CHANNEL_X == 1)
		cancon = 0x00;			//Need to test this works.
		canstat = 0x00;
	#endif

	#if (MX_CAN_CHANNEL_X == 2)

	#endif
}

CALFUNCTION(void, FC_CAL_Internal_CAN_Send_Buffer_, (MX_UINT8 buffer))
{
	#if (MX_CAN_CHANNEL_X == 1)
		unsigned int timeout = 0xFFFF;
		switch (buffer)
		{
			case 0:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x08;
				st_bit(rxb0con, TXREQ);							//Set Message Transmission Flag
				while (ts_bit(rxb0con, TXREQ) && timeout)		//Wait for message to send
					timeout = timeout - 1;
				break;
			case 1:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x06;
				st_bit(rxb0con, TXREQ);
				while (ts_bit(rxb0con, TXREQ) && timeout)
					timeout = timeout - 1;
				break;
			case 2:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x04;
				st_bit(rxb0con, TXREQ);
				while (ts_bit(rxb0con, TXREQ) && timeout)
					timeout = timeout - 1;
				break;
		}
	#endif

	#if (MX_CAN_CHANNEL_X == 2)

	#endif
}

CALFUNCTION(MX_UINT8, FC_CAL_Internal_CAN_CheckRX_, (MX_UINT8 buffer))
{
	#if (MX_CAN_CHANNEL_X == 1)
		if (buffer == 0)
		{
			cancon = cancon & 0xE0;
			//cancon = cancon | 0x0C;	//Rx Buffer 0
		}
		else
		{
			cancon = cancon & 0xE0;
			cancon = cancon | 0x0A; 	//Rx Buffer 1
		}

		if (ts_bit(rxb0con, RXFUL))
		{
			CAN_MSG_DLC = rxb0dlc & 0b00001111;	// Retrieve message length.
			CAN_MSG_ID[0] = rxb0sidh;
			CAN_MSG_ID[1] = rxb0sidl;
			if(CAN_MSG_DLC > 0)
			{
				CAN_MSG_DATA[0] = rxb0d0;
				if(CAN_MSG_DLC > 1)
				{
					CAN_MSG_DATA[1] = rxb0d1;
					if(CAN_MSG_DLC > 2)
					{
						CAN_MSG_DATA[2] = rxb0d2;
						if(CAN_MSG_DLC > 3)
						{
							CAN_MSG_DATA[3] = rxb0d3;
							if(CAN_MSG_DLC > 4)
							{
								CAN_MSG_DATA[4] = rxb0d4;
								if(CAN_MSG_DLC > 5)
								{
									CAN_MSG_DATA[5] = rxb0d5;
									if(CAN_MSG_DLC > 6)
									{
										CAN_MSG_DATA[6] = rxb0d6;
										if(CAN_MSG_DLC > 7)
										{
											CAN_MSG_DATA[7] = rxb0d7;
										}
									}
								}
							}
						}
					}
				}
			}
			cr_bit(rxb0con, RXFUL);
			return 1;
		}
		else
			return 0;
	#endif

	#if (MX_CAN_CHANNEL_X == 2)

	#endif
}

CALFUNCTION(void, FC_CAL_Internal_CAN_Set_TX_Data_, (MX_UINT8 buffer, MX_UINT8 data_cnt, MX_UINT8 d0, MX_UINT8 d1, MX_UINT8 d2, MX_UINT8 d3, MX_UINT8 d4, MX_UINT8 d5, MX_UINT8 d6, MX_UINT8 d7))
{
	#if (MX_CAN_CHANNEL_X == 1)
		switch(buffer)
		{
			case 0:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x08;	//Tx Buffer 0
				break;
			case 1:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x06;	//Tx Buffer 1
				break;
			case 2:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x04;	//Tx Buffer 2
				break;
		}

		rxb0dlc = data_cnt;				// Set data count
		if (data_cnt > 0)
		{
		   rxb0d0 = d0;
		   if (data_cnt > 1)
		   {
			   rxb0d1 = d1;
			   if (data_cnt > 2)
			   {
				   rxb0d2 = d2;
				   if (data_cnt > 3)
				   {
					   rxb0d3 = d3;
					   if (data_cnt > 4)
					   {
						   rxb0d4 = d4;
						   if (data_cnt > 5)
						   {
							   rxb0d5 = d5;
							   if (data_cnt > 6)
							   {
								   rxb0d6 = d6;
								   if (data_cnt > 7)
								   {
									   rxb0d7 = d7;
								   }
							   }
						   }
					   }
				   }
			   }
		   }
		}
	#endif

	#if (MX_CAN_CHANNEL_X == 2)

	#endif
}

CALFUNCTION(void, FC_CAL_Internal_CAN_Set_TX_Std_ID_, (MX_UINT8 buffer, MX_UINT8 hi, MX_UINT8 lo))
{
	#if (MX_CAN_CHANNEL_X == 1)
		switch(buffer)
		{
			case 0:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x08;
				txb0sidh = hi;			//set the Hi ID value
				txb0sidl = lo;			//set the Lo ID value
				break;
			case 1:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x06;
				txb1sidh = hi;			//set the Hi ID value
				txb1sidl = lo;			//set the Lo ID value
				break;
			case 2:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x04;
				txb2sidh = hi;			//set the Hi ID value
				txb2sidl = lo;			//set the Lo ID value
				break;
		}
	#endif

	#if (MX_CAN_CHANNEL_X == 2)

	#endif
}

CALFUNCTION(void, FC_CAL_Internal_CAN_Set_TX_Exd_ID_, (MX_UINT8 buffer, MX_UINT8 id3, MX_UINT8 id2, MX_UINT8 id1, MX_UINT8 id0))
{
	#if (MX_CAN_CHANNEL_X == 1)
		switch(buffer)
		{
			case 0:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x08;
				txb0sidh = id1;			//set the Hi ID value
				txb0sidl = id0;			//set the Lo ID value
				txb0eidh = id3;			//set the Hi ID value
				txb0eidl = id2;			//set the Lo ID value
				break;
			case 1:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x06;
				txb1sidh = id1;			//set the Hi ID value
				txb1sidl = id0;			//set the Lo ID value
				txb1eidh = id3;			//set the Hi ID value
				txb1eidl = id2;			//set the Lo ID value
				break;
			case 2:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x04;
				txb2sidh = id1;			//set the Hi ID value
				txb2sidl = id0;			//set the Lo ID value
				txb2eidh = id3;			//set the Hi ID value
				txb2eidl = id2;			//set the Lo ID value
				break;
		}
	#endif

	#if (MX_CAN_CHANNEL_X == 2)

	#endif
}

CALFUNCTION(void, FC_CAL_Internal_CAN_Set_Std_RX_Filter_, (MX_UINT8 filter, MX_UINT8 hi, MX_UINT8 lo))
{
	#if (MX_CAN_CHANNEL_X == 1)
		switch(filter)
		{
			case 0:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x08;
				rxf0sidh = hi;				//set the Hi ID value
				rxf0sidl = lo;				//set the Lo ID value
				break;
			case 1:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x06;
				rxf1sidh = hi;				//set the Hi ID value
				rxf1sidl = lo;				//set the Lo ID value
				break;
			case 2:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x04;
				rxf2sidh = hi;				//set the Hi ID value
				rxf2sidl = lo;				//set the Lo ID value
				break;
		}
	#endif

	#if (MX_CAN_CHANNEL_X == 2)

	#endif
}

CALFUNCTION(void, FC_CAL_Internal_CAN_Set_Exd_RX_Filter_, (MX_UINT8 filter, MX_UINT8 id3, MX_UINT8 id2, MX_UINT8 id1, MX_UINT8 id0))
{
	#if (MX_CAN_CHANNEL_X == 1)
		switch(filter)
		{
			case 0:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x08;
				rxf0sidh = id1;			//set the Hi ID value
				rxf0sidl = id0;			//set the Lo ID value
				rxf0eidh = id3;
				rxf0eidl = id2;
				break;
			case 1:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x06;
				rxf1sidh = id1;			//set the Hi ID value
				rxf1sidl = id0;			//set the Lo ID value
				rxf1eidh = id3;
				rxf1eidl = id2;
				break;
			case 2:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x04;
				rxf2sidh = id1;			//set the Hi ID value
				rxf2sidl = id0;			//set the Lo ID value
				rxf2eidh = id3;
				rxf2eidl = id2;
				break;
		}
	#endif

	#if (MX_CAN_CHANNEL_X == 2)

	#endif
}

CALFUNCTION(void, FC_CAL_Internal_CAN_Set_Std_RX_Mask_, (MX_UINT8 filter, MX_UINT8 hi, MX_UINT8 lo))
{
	#if (MX_CAN_CHANNEL_X == 1)
		switch(filter)
		{
			case 0:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x08;
				rxf0sidh = hi;			//set the Hi ID value
				rxf0sidl = lo;			//set the Lo ID value
				break;
			case 1:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x06;
				rxf1sidh = hi;			//set the Hi ID value
				rxf1sidl = lo;			//set the Lo ID value
				break;
			case 2:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x04;
				rxf2sidh = hi;			//set the Hi ID value
				rxf2sidl = lo;			//set the Lo ID value
				break;
		}
	#endif

	#if (MX_CAN_CHANNEL_X == 2)

	#endif
}

CALFUNCTION(void, FC_CAL_Internal_CAN_Set_Exd_RX_Mask, (MX_UINT8 filter, MX_UINT8 id3, MX_UINT8 id2, MX_UINT8 id1, MX_UINT8 id0))
{
	#if (MX_CAN_CHANNEL_X == 1)
		switch(filter)
		{
			case 0:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x08;
				rxf0sidh = id1;		//set the Hi ID value
				rxf0sidl = id0;		//set the Lo ID value
				rxf0eidh = id3;
				rxf0eidl = id2;
				break;
			case 1:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x06;
				rxf1sidh = id1;		//set the Hi ID value
				rxf1sidl = id0;		//set the Lo ID value
				rxf1eidh = id3;
				rxf1eidl = id2;
				break;
			case 2:
				cancon = cancon & 0xE0;
				cancon = cancon | 0x04;
				rxf2sidh = id1;		//set the Hi ID value
				rxf2sidl = id0;		//set the Lo ID value
				rxf2eidh = id3;
				rxf2eidl = id2;
				break;
		}
	#endif

	#if (MX_CAN_CHANNEL_X == 2)

	#endif
}

#endif
