/*********************************************************************
 *                  Flowcode CAL EEPROM File
 *
 * File: PIC_CAL_EEPROM.c
 *
 * (c) 2011 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 210911 | BR | Created
 */



//EEPROM Function Prototypes
MX_UINT8 FC_CAL_EE_Read (MX_UINT16 Address);
void FC_CAL_EE_Write (MX_UINT16 Address, MX_UINT8 Data);



MX_UINT8 FC_CAL_EE_Read (MX_UINT16 Address)
{
	char data = 0;
	#ifdef MX_EE
		#ifdef _BOOSTC

			#ifdef EEADR
			  #ifndef MX_EEADR
				#define MX_EEADR eeadr
			  #endif
			#endif
			#ifdef EEADRL
			  #ifndef MX_EEADR
				#define MX_EEADR eeadrl
			  #endif
			#endif

			#ifdef EEDATA
			  #ifndef MX_EEDATA
				#define MX_EEDATA eedata
			  #endif
			#endif
			#ifdef EEDATAL
			  #ifndef MX_EEDATA
				#define MX_EEDATA eedatal
			  #endif
			#endif
			#ifdef EEDATL
			  #ifndef MX_EEDATA
				#define MX_EEDATA eedatl
			  #endif
			#endif

			#ifndef MX_EEADR	//Catch Error
				#error "EEPROM error - MX_EEADR not set"
			#endif
			#ifndef MX_EEDATA
				#error "EEPROM error - MX_EEDATA not set"
			#endif

			MX_EEADR = Address & 0xff;
			#if (MX_EE_SIZE > 256)
				eeadrh = (Address & 0xff00) >> 8;
			#endif
			#ifdef MX_EE_TYPE2
				clear_bit(eecon1, EEPGD);
			#endif
			#ifdef MX_EE_TYPE3
				clear_bit(eecon1, EEPGD);
				clear_bit(eecon1, CFGS);
			#endif
			set_bit(eecon1, RD);
			data = MX_EEDATA;
		#endif
		#ifdef HI_TECH_C
			data = eeprom_read(Address);
		#endif
	#else
		#error "Chip does not have EEPROM memory"
	#endif
	return (data);
}

void FC_CAL_EE_Write (MX_UINT16 Address, MX_UINT8 Data)
{
	#ifdef MX_EE
		#ifdef _BOOSTC

			#ifdef EEADR
			  #ifndef MX_EEADR
				#define MX_EEADR eeadr
			  #endif
			#endif
			#ifdef EEADRL
			  #ifndef MX_EEADR
				#define MX_EEADR eeadrl
			  #endif
			#endif

			#ifdef EEDATA
			  #ifndef MX_EEDATA
				#define MX_EEDATA eedata
			  #endif
			#endif
			#ifdef EEDATAL
			  #ifndef MX_EEDATA
				#define MX_EEDATA eedatal
			  #endif
			#endif
			#ifdef EEDATL
			  #ifndef MX_EEDATA
				#define MX_EEDATA eedatl
			  #endif
			#endif

			#ifndef MX_EEADR	//Catch Error
				#error "EEPROM error - MX_EEADR not set"
			#endif
			#ifndef MX_EEDATA
				#error "EEPROM error - MX_EEDATA not set"
			#endif

			char bInterruptsEnabled;
			//wait for previous EE writes to complete...
			while (test_bit(eecon1, WR));
			MX_EEADR = Address & 0xff;
			#if (MX_EE_SIZE > 256)
				eeadrh = (Address & 0xff00) >> 8;
			#endif
			MX_EEDATA = Data;
			#ifdef MX_EE_TYPE2
				clear_bit(eecon1, EEPGD);
			#endif
			#ifdef MX_EE_TYPE3
				clear_bit(eecon1, EEPGD);
				clear_bit(eecon1, CFGS);
			#endif
			set_bit(eecon1, WREN);
			bInterruptsEnabled = test_bit(intcon, GIE);
			clear_bit(intcon, GIE);			//Disable Interrupts
			eecon2 = 0x55;
			eecon2 = 0xAA;
			set_bit(eecon1, WR);
			while (test_bit(eecon1, WR));	//wait for EE write to complete...
			if (bInterruptsEnabled)
				set_bit(intcon, GIE);		//Re-enable Interrupts
			clear_bit(eecon1, WREN);
		#endif
		#ifdef HI_TECH_C
			eeprom_write(Address, Data);
		#endif
	#else
		#error "Chip does not have EEPROM memory"
	#endif
}

