/*********************************************************************
 *                  Flowcode CAL Float File
 *
 * File: PIC_CAL_Float.c
 *
 * (c) 2011 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 010911 | BR | Created
 *
 *
 *
 */


char isinf(float f);

#ifdef HI_TECH_C

  #define flt_add(a, b) ((a) + (b))			  	// Binary arithmetic
  #define flt_sub(a, b) ((a) - (b))
  #define flt_mul(a, b) ((a) * (b))
  #define flt_div(a, b) ((a) / (b))
  #define flt_rem(a, b) ((a) % (b))

  #define flt_neg(a) (-(a))					  	//Unary arithmetic

  #define flt_eq(a, b) ((a) == (b))			  	//Comparisons
  #define flt_ne(a, b) ((a) != (b))
  #define flt_ge(a, b) ((a) >= (b))
  #define flt_gt(a, b) ((a) > (b))
  #define flt_le(a, b) ((a) <= (b))
  #define flt_lt(a, b) ((a) < (b))

  #define flt_toi(a)   ((int)(a))			  	//Assignment
  #define flt_fromi(a) ((float)(a))

  // Rounding functions - nearest, down, up
  #define flt_round(f) ((int)((f) + 0.5))
  #define flt_floor(f) ((int)(f))
  #define flt_ceil(f)  ((int)(f) + 1)
#else

  #define flt_add(a, b) float32_add((a), (b))  	// Binary arithmetic
  #define flt_sub(a, b) float32_sub((a), (b))
  #define flt_mul(a, b) float32_mul((a), (b))
  #define flt_div(a, b) float32_div((a), (b))
  #define flt_rem(a, b) float32_rem((a), (b))

  #define flt_neg(a) float32_sub((0.0), (a)) 	//Unary arithmetic - With op0 always 0.0

  #define flt_eq(a, b) float32_eq((a), (b))		//Comparisons
  #define flt_ne(a, b) !float32_eq((a), (b))
  #define flt_ge(a, b) float32_ge((a), (b))
  #define flt_gt(a, b) float32_gt((a), (b))
  #define flt_le(a, b) float32_le((a), (b))
  #define flt_lt(a, b) float32_lt((a), (b))

  #define flt_toi(a)   float32_to_int32((a))	//Assignment
  #define flt_fromi(a) float32_from_int32((a))

  // Rounding functions - nearest, down, up
  int flt_round(float f)
  {
      float_rounding_mode = float_round_nearest_even;
      return float32_to_int32(f);
  }
  int flt_floor(float f)
  {
      float_rounding_mode = float_round_down;
      return float32_to_int32(f);
  }
  int flt_ceil(float f)
  {
      float_rounding_mode = float_round_up;
      return float32_to_int32(f);
  }
#endif

char isinf(float f)
{
      char* b;
      b = (char*)&f;
      if ((b[0] == 0)   &&
          (b[1] == 0)   &&
          (b[2] == 128) &&
          ((b[3] & 127) == 127))
      {
            return (255);
      }
      return (0);
}

