/*********************************************************************
 *                 Flowcode Bluetooth Component Code
 *
 * File: Bluetooth.c
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 240407 | ST | Created
 * 190607 | BR | Externalised
 * 120707 | ST | Modified to work with CTL file
 * 250907 | SK | Modified delay constants and low-level read/write routines
 * 031007 | ST | Component now chooses 16 or 64 divide for BaudRate ( %j = bUse16)
 * 240309 | BR | Moved private functions from AddDefines section
 * 130509 | BR | Added % definitions to DefinesCode section
 * 100909 | BR | Added HiTECH compatible definitions
 * 131109 | BR | Fixed #pragma error HITECH problem
 * 230311 | BR | Changed code to work for hitech and boostc without ifdefs
 * 210611 | SK | Addidion of MIAC System support
 * 050911 | BR | Converted to Universal Bluetooth Component Code File
 * 011211 | BR | Updated FC_CAL_Bit functions
 * 021211 | BR | Converted to new dynamic defines mechanism
 ********************************************************************/

/*********************************************************************

 Return & parameter types:
   void
   MX_UINT8
   MX_SINT16
   MX_STRING

 Pin directions:
   0 = OUTPUT
   2 = INPUT
   3 = BIDIRECTIONAL

**********************************************************************

[Settings]
CLSID={53D2B7AA-BFF9-4CEE-B8BA-428EF45DA86F}
IsAnalogue=0
MultipleAllowed=0
Description=Bluetooth Flowcode Component

**********************************************************************

[Port]
DesiredPort=0
PortMandatory=0
SamePortAsID=0

[Pins]
Count=2
1=RTS
2=CTS

[PinPort]
# PORTA = 0
# PORTB = 1
# PORTC = 2
# PORTD = 3
# PORTE = 4
1=0
2=0

[PinDesiredBit]
1=0
2=4

[PinDirection]
# DIR_INPUT = 2
# DIR_OUTPUT = 0
# DIR_BIDIRECTIONAL = 3
1=0
2=2

[PinMustUsePort]
# boolean
1=0
2=0

[SamePortAsPinX]
# -1 = any pin
# otherwise, the pin number
1=-1
2=0

[PinMustUsePin]
# boolean
1=0
2=0

[PinValue]
# boolean (0 = off, 1 = on)
1=0
2=0

**********************************************************************

//Bluetooth0: //Macro function declarations

[MacroNames]
Count=10
1=Blu_MaskFromBit
2=Blu_Send_Byte
3=Blu_Read_Byte
4=Initialize
5=CreateCommand
6=SendCommand
7=WaitForResponse
8=StringReceive
9=StringRead
10=SendScript

[MacroReturns]
1=MX_UINT8
2=void
3=MX_UINT8
4=void
5=MX_UINT8
6=MX_UINT8
7=MX_UINT8
8=MX_UINT8
9=MX_UINT8
10=MX_UINT8

[MacroIsPrivate]
1=1
2=1
3=1
4=0
5=0
6=0
7=0
8=0
9=0
10=0

[MacroParameters_Blu_MaskFromBit]
Count=1
1=mybit

[MacroParamTypes_Blu_MaskFromBit]
1=MX_UINT8

[MacroParameters_Blu_Send_Byte]
Count=1
1=byte

[MacroParamTypes_Blu_Send_Byte]
1=MX_UINT8

[MacroParameters_Blu_Read_Byte]
Count=1
1=timeout

[MacroParamTypes_Blu_Read_Byte]
1=MX_UINT8

[MacroParameters_Initialize]
Count=0

[MacroParamTypes_Initialize]


[MacroParameters_CreateCommand]
Count=1
1=ch

[MacroParamTypes_CreateCommand]
1=MX_UINT8

[MacroParameters_SendCommand]
Count=2
1=bExpectEcho
2=bSendCR

[MacroParamTypes_SendCommand]
1=MX_UINT8
2=MX_UINT8

[MacroParameters_WaitForResponse]
Count=2
1=response_code
2=timeout

[MacroParamTypes_WaitForResponse]
1=MX_UINT8
2=MX_UINT8

[MacroParameters_StringReceive]
Count=0

[MacroParamTypes_StringReceive]


[MacroParameters_StringRead]
Count=1
1=idx

[MacroParamTypes_StringRead]
1=MX_UINT8

[MacroParameters_SendScript]
Count=1
1=idx

[MacroParamTypes_SendScript]
1=MX_UINT8

 ********************************************************************/

/********************************************************************
 * ADDITIONAL CODE
 ********************************************************************/

/*DefinesCode_Start*/

/**** Macro Substitutions ****
a = Unique Reference
b = Uart Channel		(0-software / 1-4 hardware)
c = tx pin
d = tx port
e = rx pin
f = rx port
g = flow control		(0-Off / 1-On)
h = cts pin
i = cts port
j = rts pin
k = rts port
l = baud rate
m = Number of scripts
n = Script 1
o = Script 2
p = Script 3
q = Script 4
******************************/

//Definitions for Bluetooth UART slot allocation
#ifndef	MX_UART_REF1
	#ifndef %a_MX_UART_UREF
		#define MX_UART_REF1
		#define %a_MX_UART_UREF			1
		#define MX_UART_CHANNEL_1		%b
		#define MX_UART_TX_PIN_1		%c
		#define MX_UART_TX_PORT_1		port%d
		#define MX_UART_TX_TRIS_1		tris%d
		#define MX_UART_RX_PIN_1		%e
		#define MX_UART_RX_PORT_1		port%f
		#define MX_UART_RX_TRIS_1		tris%f
		#define MX_UART_FLOWEN_1		%g
		#define MX_UART_CTS_PIN_1		%h
		#define MX_UART_CTS_PORT_1		port%i
		#define MX_UART_CTS_TRIS_1		tris%i
		#define MX_UART_RTS_PIN_1		%j
		#define MX_UART_RTS_PORT_1		port%k
		#define MX_UART_RTS_TRIS_1		tris%k
		#define MX_UART_BAUD_1			%l
		#define MX_UART_DBITS_1			8
		#define MX_UART_RETURN_1		0
		#define MX_UART_ECHO_1			0
		#define MX_UART_INT_1			0
	#endif
#else
	#ifndef	MX_UART_REF2
		#ifndef %a_MX_UART_UREF
			#define MX_UART_REF2
			#define %a_MX_UART_UREF			2
			#define MX_UART_CHANNEL_2		%b
			#define MX_UART_TX_PIN_2		%c
			#define MX_UART_TX_PORT_2		port%d
			#define MX_UART_TX_TRIS_2		tris%d
			#define MX_UART_RX_PIN_2		%e
			#define MX_UART_RX_PORT_2		port%f
			#define MX_UART_RX_TRIS_2		tris%f
			#define MX_UART_FLOWEN_2		%g
			#define MX_UART_CTS_PIN_2		%h
			#define MX_UART_CTS_PORT_2		port%i
			#define MX_UART_CTS_TRIS_2		tris%i
			#define MX_UART_RTS_PIN_2		%j
			#define MX_UART_RTS_PORT_2		port%k
			#define MX_UART_RTS_TRIS_2		tris%k
			#define MX_UART_BAUD_2			%l
			#define MX_UART_DBITS_2			8
			#define MX_UART_RETURN_2		0
			#define MX_UART_ECHO_2			0
			#define MX_UART_INT_2			0
		#endif
	#else
		#ifndef	MX_UART_REF3
			#ifndef %a_MX_UART_UREF
				#define MX_UART_REF3
				#define %a_MX_UART_UREF			3
				#define MX_UART_CHANNEL_3		%b
				#define MX_UART_TX_PIN_3		%c
				#define MX_UART_TX_PORT_3		port%d
				#define MX_UART_TX_TRIS_3		tris%d
				#define MX_UART_RX_PIN_3		%e
				#define MX_UART_RX_PORT_3		port%f
				#define MX_UART_RX_TRIS_3		tris%f
				#define MX_UART_FLOWEN_3		%g
				#define MX_UART_CTS_PIN_3		%h
				#define MX_UART_CTS_PORT_3		port%i
				#define MX_UART_CTS_TRIS_3		tris%i
				#define MX_UART_RTS_PIN_3		%j
				#define MX_UART_RTS_PORT_3		port%k
				#define MX_UART_RTS_TRIS_3		tris%k
				#define MX_UART_BAUD_3			%l
				#define MX_UART_DBITS_3			8
				#define MX_UART_RETURN_3		0
				#define MX_UART_ECHO_3			0
				#define MX_UART_INT_3			0
			#endif
		#else
			#ifndef	MX_UART_REF4
				#ifndef %a_MX_UART_UREF
					#define MX_UART_REF4
					#define %a_MX_UART_UREF			4
					#define MX_UART_CHANNEL_4		%b
					#define MX_UART_TX_PIN_4		%c
					#define MX_UART_TX_PORT_4		port%d
					#define MX_UART_TX_TRIS_4		tris%d
					#define MX_UART_RX_PIN_4		%e
					#define MX_UART_RX_PORT_4		port%f
					#define MX_UART_RX_TRIS_4		tris%f
					#define MX_UART_FLOWEN_4		%g
					#define MX_UART_CTS_PIN_4		%h
					#define MX_UART_CTS_PORT_4		port%i
					#define MX_UART_CTS_TRIS_4		tris%i
					#define MX_UART_RTS_PIN_4		%j
					#define MX_UART_RTS_PORT_4		port%k
					#define MX_UART_RTS_TRIS_4		tris%k
					#define MX_UART_BAUD_4			%l
					#define MX_UART_DBITS_4			8
					#define MX_UART_RETURN_4		0
					#define MX_UART_ECHO_4			0
					#define MX_UART_INT_4			0
				#endif
			#endif
		#endif
	#endif
#endif

#define %a_UART_Init		CAL_APPEND(FC_CAL_UART_Init_, %a_MX_UART_UREF)
#define %a_UART_Send		CAL_APPEND(FC_CAL_UART_Send_, %a_MX_UART_UREF)
#define %a_UART_Receive		CAL_APPEND(FC_CAL_UART_Receive_, %a_MX_UART_UREF)

extern void %a_UART_Init();
extern void %a_UART_Send(MX_UINT16 nChar);
extern MX_SINT16 %a_UART_Receive(MX_UINT8 nTimeout);

#define %a_BLU_SCRIPT_COUNT	%m

#if ( %m >= 1)
  const MX_STRING %a_BLU_SCRIPT_1 = "%n";
#endif
#if ( %m >= 2)
  const MX_STRING %a_BLU_SCRIPT_2 = "%o";
#endif
#if ( %m >= 3)
  const MX_STRING %a_BLU_SCRIPT_3 = "%p";
#endif
#if ( %m >= 4)
  const MX_STRING %a_BLU_SCRIPT_4 = "%q";
#endif

#define %a_BLU_STRING_ARRAY_SIZE 		32		//see "at info.txt" for discussion about the string array size and why we have settled on 32.
#define %a_BLU_STRING_RESPONSE_SIZE 	14

#define %a_BLU_RESPONSE_OK           1
#define %a_BLU_RESPONSE_ERROR        2
#define %a_BLU_RESPONSE_CONNECT      3
#define %a_BLU_RESPONSE_NOCARRIER    4
#define %a_BLU_RESPONSE_AUDIO        5
#define %a_BLU_RESPONSE_PAIR         6
#define %a_BLU_RESPONSE_RING         7

#define %a_BLU_CHAR_CR              13
#define %a_BLU_CHAR_LF              10

#define %a_RS_STATUS_OK              0
#define %a_RS_STATUS_TIMEOUT         1
#define %a_RS_STATUS_LOOP            2
#define %a_RS_STATUS_FERR            3
#define %a_RS_STATUS_OERR            4

//Timeout within the <BT_Send_Command> function (ms)
#define %a_BLU_SENDCMD_TIMEOUT		 75

//Timeout within the <BT_Wait_For_Response> function (ms)
#define %a_BLU_RESPONSE_TIMEOUT     75

//Timeout within the <BT_String_Receive> function (ms)
#define %a_BLU_STRINGRX_TIMEOUT     75

//Timeout within the <BT_Send_Script> function (ms)
#define %a_BLU_SENDSCRIPT_TIMEOUT   75

//common globals
MX_UINT8 %a_BLU_STRING_ARRAY_TXPTR;
MX_UINT8 %a_BLU_STRING_ARRAY_RXPTR;
MX_UINT8 %a_BLU_STRING_ARRAY[%a_BLU_STRING_ARRAY_SIZE];
MX_UINT8 %a_BLU_RESPONSE_ARRAY[%a_BLU_STRING_RESPONSE_SIZE];

MX_UINT8 %a_BLU_RX_STATUS;
MX_UINT8 %a_BLU232_TIMEOUT;

/*DefinesCode_End*/

/*InitialisationCode_Start*/
/*InitialisationCode_End*/

/*InterruptCode_Start*/
/*InterruptCode_End*/

/********************************************************************
 * FUNCTIONS
 ********************************************************************/

//Macro implementations

MX_UINT8 Blu_MaskFromBit(MX_UINT8 mybit)
{
/*Macro_Blu_MaskFromBit_Start*/
    MX_UINT8 retVal = 0;
    switch (mybit)
    {
        case 0:
            retVal = 0x01;
            break;

        case 1:
            retVal = 0x02;
            break;

        case 2:
            retVal = 0x04;
            break;

        case 3:
            retVal = 0x08;
            break;

        case 4:
            retVal = 0x10;
            break;

        case 5:
            retVal = 0x20;
            break;

        case 6:
            retVal = 0x40;
            break;

        case 7:
            retVal = 0x80;
            break;
    }
    return (retVal);
/*Macro_Blu_MaskFromBit_End*/
}


void Blu_Send_Byte(MX_UINT8 byte)
{
/*Macro_Blu_Send_Byte_Start*/

	%a_UART_Send (byte);

/*Macro_Blu_Send_Byte_End*/
}


MX_UINT8 Blu_Read_Byte(MX_UINT8 timeout)
{
/*Macro_Blu_Read_Byte_Start*/

	return (%a_UART_Receive(timeout));

/*Macro_Blu_Read_Byte_End*/
}


void Initialize()
{
/*Macro_Initialize_Start*/

	%a_UART_Init();		//Call initialise function

/*Macro_Initialize_End*/
}



MX_UINT8 CreateCommand(MX_UINT8 ch)
{
/*Macro_CreateCommand_Start*/
    if ( %a_BLU_STRING_ARRAY_TXPTR < %a_BLU_STRING_ARRAY_SIZE)
    {
        %a_BLU_STRING_ARRAY[%a_BLU_STRING_ARRAY_TXPTR] = ch;
        %a_BLU_STRING_ARRAY_TXPTR++;
        return(1);
    }
    else
    {
        return(0);
    }
/*Macro_CreateCommand_End*/
}



MX_UINT8 SendCommand(MX_UINT8 bExpectEcho, MX_UINT8 bSendCR)
{
/*Macro_SendCommand_Start*/
    MX_UINT8 idx = 0;
    MX_UINT8 dummy;

    if ( %a_BLU_STRING_ARRAY_TXPTR > 0)
    {
        while (idx < %a_BLU_STRING_ARRAY_TXPTR)
        {
            Blu_Send_Byte( %a_BLU_STRING_ARRAY[idx]);
            //Wdt_Delay_Ms(1);

            if (bExpectEcho != 0)
            {
                //read echoed MX_UINT8acter
                dummy = Blu_Read_Byte( %a_BLU_SENDCMD_TIMEOUT);
            }

            idx++;
            //Wdt_Delay_Ms(2);
        }

        if (bSendCR != 0)
        {
            Blu_Send_Byte(0x0D);            //send the terminating CR
            //Wdt_Delay_Ms(1);

            if (bExpectEcho != 0)
            {
                //read echoed MX_UINT8acter
                dummy = Blu_Read_Byte( %a_BLU_SENDCMD_TIMEOUT);
            }
        }

        %a_BLU_STRING_ARRAY_TXPTR = 0;
        return (1);

    }
    else
    {
        return (0);
    }
/*Macro_SendCommand_End*/
}



MX_UINT8 WaitForResponse(MX_UINT8 response_code, MX_UINT8 timeout)
{
/*Macro_WaitForResponse_Start*/
    //returns zero for OK
    //returns 0xFF for timeout or an illegal response code
    //returns the number of MX_UINT8acters received if an unexpected response is received
    MX_UINT8 resp_size = 0;
    MX_UINT8 ch;
    MX_UINT8 rx_ptr;
    MX_UINT8 crlf_flags = 0;
    MX_UINT8 unexpected = 0;
    //responses always start with a <cr><lf> combination
    %a_BLU_RESPONSE_ARRAY[resp_size++] = %a_BLU_CHAR_CR;
    %a_BLU_RESPONSE_ARRAY[resp_size++] = %a_BLU_CHAR_LF;

    switch (response_code)
    {
        case %a_BLU_RESPONSE_OK:
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'O';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'K';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = %a_BLU_CHAR_CR;
            %a_BLU_RESPONSE_ARRAY[resp_size++] = %a_BLU_CHAR_LF;
            break;

        case %a_BLU_RESPONSE_ERROR:
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'E';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'R';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'R';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'O';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'R';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = ' ';
            break;

        case %a_BLU_RESPONSE_CONNECT:
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'C';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'O';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'N';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'N';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'E';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'C';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'T';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = ' ';
            break;

        case %a_BLU_RESPONSE_NOCARRIER:
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'N';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'O';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = ' ';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'C';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'A';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'R';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'R';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'I';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'E';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'R';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = %a_BLU_CHAR_CR;
            %a_BLU_RESPONSE_ARRAY[resp_size++] = %a_BLU_CHAR_LF;
            break;

        case %a_BLU_RESPONSE_AUDIO:
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'A';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'U';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'D';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'I';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'O';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = ' ';
            break;

        case %a_BLU_RESPONSE_PAIR:
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'P';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'A';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'I';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'R';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = ' ';
            break;

        case %a_BLU_RESPONSE_RING:
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'R';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'I';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'N';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = 'G';
            %a_BLU_RESPONSE_ARRAY[resp_size++] = ' ';
            break;

        default:
            //unknown response - error
            return (0xFF);
    }
    %a_BLU_STRING_ARRAY_RXPTR = 0;
    rx_ptr = 0;

    while (crlf_flags < 0x03)
    {
        ch = Blu_Read_Byte( %a_BLU_RESPONSE_TIMEOUT);

        //if there was a timeout, we need to carry on checking...
        while ( %a_BLU_RX_STATUS != %a_RS_STATUS_OK)
        {
            timeout--;
            if (timeout == 0)
            {
                return (0xFF);
            }

            //TODO: may need a delay here to extend the complete timeout value
            //Wdt_Delay_Ms(1);
            ch = Blu_Read_Byte( %a_BLU_STRINGRX_TIMEOUT);
        }
        if (ch == 0x0D)
        {
            crlf_flags = crlf_flags | 0x01;

        }
        else if (ch == 0x0A)
        {
            crlf_flags = crlf_flags | 0x02;

        }
        else
        {
            %a_BLU_STRING_ARRAY[%a_BLU_STRING_ARRAY_RXPTR] = ch;
            %a_BLU_STRING_ARRAY_RXPTR++;
            if ( %a_BLU_STRING_ARRAY_RXPTR >= %a_BLU_STRING_ARRAY_SIZE)
            {
                //max buffer size has been reached
                return ( %a_BLU_STRING_ARRAY_RXPTR);
            }
        }
        //we have now received a legitimate MX_UINT8acter
        //reset the clrf flags and check for expected input
        //(if not previously received unexpected MX_UINT8s)
        if (unexpected == 0)
        {
            crlf_flags = 0;

            //is it expected?
            if (ch != %a_BLU_RESPONSE_ARRAY[rx_ptr])
            {
                unexpected = 1;
            }
            //increment the pointer
            rx_ptr++;
        }
        //have we finished looking for the "expected" string
        if ((unexpected == 0) && (rx_ptr == resp_size))
        {
            //yes - so return zero
            return (0);
        }
    }
    //if we get here, we have received an unexpected response
    return ( %a_BLU_STRING_ARRAY_RXPTR);
/*Macro_WaitForResponse_End*/
}



MX_UINT8 StringReceive()
{
/*Macro_StringReceive_Start*/
    //returns the number of MX_UINT8acters received
    MX_UINT8 ch;
    MX_UINT8 crlf_flags = 0;         //when a cr and lf have been received, this will be > 3
    MX_UINT8 timeout = 0;

    %a_BLU_STRING_ARRAY_RXPTR = 0;
    %a_BLU_RX_STATUS = %a_RS_STATUS_OK;

    while (crlf_flags <= 0x02)
    {
        ch = Blu_Read_Byte( %a_BLU_STRINGRX_TIMEOUT);

        if ( %a_BLU_RX_STATUS == %a_RS_STATUS_OK)
        {
            if (ch == 0x0D)
            {
                crlf_flags = crlf_flags | 0x01;
            }
            else if (ch == 0x0A)
            {
                crlf_flags = crlf_flags | 0x02;
            }
            else
            {
                //not a CR or LF
                %a_BLU_STRING_ARRAY[%a_BLU_STRING_ARRAY_RXPTR] = ch;
                %a_BLU_STRING_ARRAY_RXPTR++;
                //reset the timeout
                timeout = 0;
                if ( %a_BLU_STRING_ARRAY_RXPTR >= %a_BLU_STRING_ARRAY_SIZE)
                {
                    //max buffer size has been reached
                    crlf_flags = 0xFF;
                }
            }
        }
        else
        {
            timeout++;
            if (timeout > 1)
            {
                //no further bytes have been received
                crlf_flags = 0xFF;
            }
        }
    }
    return ( %a_BLU_STRING_ARRAY_RXPTR);
/*Macro_StringReceive_End*/
}



MX_UINT8 StringRead(MX_UINT8 idx)
{
/*Macro_StringRead_Start*/
    if (idx < %a_BLU_STRING_ARRAY_RXPTR)
    {
        return ( %a_BLU_STRING_ARRAY[idx]);
    }
    else
    {
        return (0);
    }
/*Macro_StringRead_End*/
}



MX_UINT8 SendScript(MX_UINT8 idx)
{
/*Macro_SendScript_Start*/
    MX_UINT8 i = 0;
    MX_UINT8 ch = 255;

    if ((idx < 1) || (idx > %a_BLU_SCRIPT_COUNT))
    {
        return (0xFF);        //error - index out of range
    }
    while (ch > 0)
    {
        switch (idx)
        {
          #if ( %a_BLU_SCRIPT_COUNT >= 1)
            case 1:
                ch = %a_BLU_SCRIPT_1[i];
                break;
          #endif

          #if ( %a_BLU_SCRIPT_COUNT >= 2)
            case 2:
                ch = %a_BLU_SCRIPT_2[i];
                break;
          #endif

          #if ( %a_BLU_SCRIPT_COUNT >= 3)
            case 3:
                ch = %a_BLU_SCRIPT_3[i];
                break;
          #endif

          #if ( %a_BLU_SCRIPT_COUNT == 4)
            case 4:
                ch = %a_BLU_SCRIPT_4[i];
                break;
          #endif

            default:
                return (0xFF);    //should not get here
        }

        if (ch > 0)
        {
            #ifdef MX_DEBUG_%a_BLU
                //FCD_LCDDisplay0_PrintASCII(ch);
            #endif

            //send the MX_UINT8acter
            Blu_Send_Byte(ch);

            //read the echo
            Blu_Read_Byte( %a_BLU_SENDSCRIPT_TIMEOUT);

            //if we sent a CR, wait for the response
            if (ch == 13)
            {
                //wait until we are receiving no more MX_UINT8s
                %a_BLU_RX_STATUS = %a_RS_STATUS_OK;
                ch = Blu_Read_Byte( %a_BLU_RESPONSE_TIMEOUT);
                while ( %a_BLU_RX_STATUS == %a_RS_STATUS_OK)
                {
                    ch = Blu_Read_Byte( %a_BLU_SENDSCRIPT_TIMEOUT);
                    #ifdef MX_DEBUG_%a_BLU
                        //FCD_LCDDisplay0_PrintASCII('.');
                    #endif
                }
            }
        }
        i++;
    }
    return (0);
/*Macro_SendScript_End*/
}