/*********************************************************************
 *                   Flowcode GSM Component Code
 *
 * File: GSM.c
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 150411 | BR | Created
 * 230711 | BR | Change to the way bytes are received to allow for better msg reception
 * 071011 | BR | Converted to Universal GSM Component Code File
 * 081111 | ST | Added RTS, TX and RX pins
 * 011211 | BR | Converted to new dynamic defines mechanism
 * 011211 | BR | Updated FC_CAL_Bit functions
 ********************************************************************/

/*********************************************************************

 Return & parameter types:
   void
   MX_UINT8
   MX_UINT16
   MX_STRING

 Pin directions:
   0 = OUTPUT
   2 = INPUT
   3 = BIDIRECTIONAL

**********************************************************************

[Settings]
CLSID={FC63F846-39C0-4BAF-BF23-9D007D092701}
IsAnalogue=0
MultipleAllowed=1
Description=GSM Flowcode Component

**********************************************************************

[Port]
DesiredPort=0
PortMandatory=0
SamePortAsID=0

[Pins]
Count=4
1=CTS
2=RTS
3=TX
4=RX

[PinPort]
# PORTA = 0
# PORTB = 1
# PORTC = 2
# PORTD = 3
# PORTE = 4
1=2
2=2
3=2
4=2

[PinDesiredBit]
1=4
2=3
3=5
4=2

[PinDirection]
# DIR_INPUT = 2
# DIR_OUTPUT = 0
# DIR_BIDIRECTIONAL = 3
1=2
2=0
3=0
4=2

[PinMustUsePort]
# boolean
1=0
2=0
3=0
4=0

[SamePortAsPinX]
# -1 = any pin
# otherwise, the pin number
1=-1
2=-1
3=-1
4=-1

[PinMustUsePin]
# boolean
1=1
2=0
3=0
4=0

[PinValue]
# boolean (0 = off, 1 = on)
1=1
2=1
3=1
4=1

**********************************************************************

[MacroNames]
Count=14
1=Initialize
2=CheckNetworkStatus
3=SendCommand
4=StringReceive
5=SendScript
6=CheckForIncoming
7=DialNumber
8=HangUpCall
9=AnswerIncomingCall
10=SendTextMessage
11=DeleteAllMessages
12=ReadString
13=GetTextNumber
14=GetTextString

[MacroReturns]
1=MX_UINT8
2=MX_UINT8
3=MX_UINT8
4=MX_UINT8
5=MX_UINT8
6=MX_UINT8
7=MX_UINT8
8=MX_UINT8
9=MX_UINT8
10=MX_UINT8
11=MX_UINT8
12=MX_STRING
13=MX_STRING
14=MX_STRING

[MacroIsPrivate]
1=0
2=0
3=0
4=0
5=0
6=0
7=0
8=0
9=0
10=0
11=1
12=0
13=0
14=0
15=0

[MacroParameters_Initialize]
Count=0

[MacroParamTypes_Initialize]


[MacroParameters_CheckNetworkStatus]
Count=0

[MacroParamTypes_CheckNetworkStatus]


[MacroParameters_SendCommand]
Count=2
1=command
2=bSendCR

[MacroParamTypes_SendCommand]
1=MX_STRING
2=MX_UINT8

[MacroParameters_StringReceive]
Count=0

[MacroParamTypes_StringReceive]


[MacroParameters_SendScript]
Count=1
1=idx

[MacroParamTypes_SendScript]
1=MX_UINT8

[MacroParameters_CheckForIncoming]
Count=0

[MacroParamTypes_CheckForIncoming]


[MacroParameters_DialNumber]
Count=1
1=number

[MacroParamTypes_DialNumber]
1=MX_STRING

[MacroParameters_HangUpCall]
Count=0

[MacroParamTypes_HangUpCall]


[MacroParameters_AnswerIncomingCall]
Count=0

[MacroParamTypes_AnswerIncomingCall]


[MacroParameters_SendTextMessage]
Count=2
1=number
2=message

[MacroParamTypes_SendTextMessage]
1=MX_STRING
2=MX_STRING

[MacroParameters_DeleteAllMessages]
Count=0

[MacroParamTypes_DeleteAllMessages]

[MacroParameters_GetTextNumber]
Count=0

[MacroParamTypes_GetTextNumber]

[MacroParameters_GetTextString]
Count=0

[MacroParamTypes_GetTextString]

 ********************************************************************/

/********************************************************************
 * ADDITIONAL CODE
 ********************************************************************/

/*DefinesCode_Start*/

/**** Macro Substitutions ****
a = Unique Reference
b = Uart Channel		(0-software / 1-4 hardware)
c = tx pin
d = tx port
e = rx pin
f = rx port
g = flow control		(0-Off / 1-On)
h = cts pin
i = cts port
j = rts pin
k = rts port
l = echo enable		(0-Off / 1-On)
m = Number of scripts
n = Script 1
o = Script 2
p = Script 3
q = Script 4
******************************/

////////////////////////////////////////////////////
// MX_DEFINES
////////////////////////////////////////////////////

#define %a_GSM_SCRIPT_COUNT		%m

#if (%a_GSM_SCRIPT_COUNT >= 1)
  ROMARRAY_S %a_GSM_SCRIPT_1 ROMARRAY_E = "%n";
#endif
#if (%a_GSM_SCRIPT_COUNT >= 2)
  ROMARRAY_S %a_GSM_SCRIPT_2 ROMARRAY_E = "%o";
#endif
#if (%a_GSM_SCRIPT_COUNT >= 3)
  ROMARRAY_S %a_GSM_SCRIPT_3 ROMARRAY_E = "%p";
#endif
#if (%a_GSM_SCRIPT_COUNT >= 4)
  ROMARRAY_S %a_GSM_SCRIPT_4 ROMARRAY_E = "%q";
#endif

//Definitions for RS232 UART slot allocation
#ifndef	MX_UART_REF1
	#define MX_UART_REF1
	#define %a_MX_UART_UREF			1
	#define MX_UART_CHANNEL_1		%b
	#define MX_UART_TX_PIN_1		%c
	#define MX_UART_TX_PORT_1		port%d
	#define MX_UART_TX_TRIS_1		tris%d
	#define MX_UART_RX_PIN_1		%e
	#define MX_UART_RX_PORT_1		port%f
	#define MX_UART_RX_TRIS_1		tris%f
	#define MX_UART_FLOWEN_1		%g
	#define MX_UART_CTS_PIN_1		%h
	#define MX_UART_CTS_PORT_1		port%i
	#define MX_UART_CTS_TRIS_1		tris%i
	#define MX_UART_RTS_PIN_1		%j
	#define MX_UART_RTS_PORT_1		port%k
	#define MX_UART_RTS_TRIS_1		tris%k
	#define MX_UART_BAUD_1			9600
	#define MX_UART_DBITS_1			8
	#define MX_UART_RETURN_1		0
	#define MX_UART_ECHO_1			%l
	#define MX_UART_INT_1			1
#else
 #ifndef	MX_UART_REF2
	#define MX_UART_REF2
	#define %a_MX_UART_UREF			2
	#define MX_UART_CHANNEL_2		%b
	#define MX_UART_TX_PIN_2		%c
	#define MX_UART_TX_PORT_2		port%d
	#define MX_UART_TX_TRIS_2		tris%d
	#define MX_UART_RX_PIN_2		%e
	#define MX_UART_RX_PORT_2		port%f
	#define MX_UART_RX_TRIS_2		tris%f
	#define MX_UART_FLOWEN_2		%g
	#define MX_UART_CTS_PIN_2		%h
	#define MX_UART_CTS_PORT_2		port%i
	#define MX_UART_CTS_TRIS_2		tris%i
	#define MX_UART_RTS_PIN_2		%j
	#define MX_UART_RTS_PORT_2		port%k
	#define MX_UART_RTS_TRIS_2		tris%k
	#define MX_UART_BAUD_2			9600
	#define MX_UART_DBITS_2			8
	#define MX_UART_RETURN_2		0
	#define MX_UART_ECHO_2			%l
	#define MX_UART_INT_2			1
 #else
  #ifndef	MX_UART_REF3
	#define MX_UART_REF3
	#define %a_MX_UART_UREF			3
	#define MX_UART_CHANNEL_3		%b
	#define MX_UART_TX_PIN_3		%c
	#define MX_UART_TX_PORT_3		port%d
	#define MX_UART_TX_TRIS_3		tris%d
	#define MX_UART_RX_PIN_3		%e
	#define MX_UART_RX_PORT_3		port%f
	#define MX_UART_RX_TRIS_3		tris%f
	#define MX_UART_FLOWEN_3		%g
	#define MX_UART_CTS_PIN_3		%h
	#define MX_UART_CTS_PORT_3		port%i
	#define MX_UART_CTS_TRIS_3		tris%i
	#define MX_UART_RTS_PIN_3		%j
	#define MX_UART_RTS_PORT_3		port%k
	#define MX_UART_RTS_TRIS_3		tris%k
	#define MX_UART_BAUD_3			9600
	#define MX_UART_DBITS_3			8
	#define MX_UART_RETURN_3		0
	#define MX_UART_ECHO_3			%l
	#define MX_UART_INT_3			1
  #else
   #ifndef	MX_UART_REF4
	#define MX_UART_REF4
	#define %a_MX_UART_UREF			4
	#define MX_UART_CHANNEL_4		%b
	#define MX_UART_TX_PIN_4		%c
	#define MX_UART_TX_PORT_4		port%d
	#define MX_UART_TX_TRIS_4		tris%d
	#define MX_UART_RX_PIN_4		%e
	#define MX_UART_RX_PORT_4		port%f
	#define MX_UART_RX_TRIS_4		tris%f
	#define MX_UART_FLOWEN_4		%g
	#define MX_UART_CTS_PIN_4		%h
	#define MX_UART_CTS_PORT_4		port%i
	#define MX_UART_CTS_TRIS_4		tris%i
	#define MX_UART_RTS_PIN_4		%j
	#define MX_UART_RTS_PORT_4		port%k
	#define MX_UART_RTS_TRIS_4		tris%k
	#define MX_UART_BAUD_4			9600
	#define MX_UART_DBITS_4			8
	#define MX_UART_RETURN_4		0
	#define MX_UART_ECHO_4			%l
	#define MX_UART_INT_4			1
   #endif
  #endif
 #endif
#endif


#define %a_UART_Init		CAL_APPEND(FC_CAL_UART_Init_, %a_MX_UART_UREF)
#define %a_UART_Send		CAL_APPEND(FC_CAL_UART_Send_, %a_MX_UART_UREF)
#define %a_UART_Receive		CAL_APPEND(FC_CAL_UART_Receive_, %a_MX_UART_UREF)

extern void %a_UART_Init();
extern void %a_UART_Send(MX_UINT16 nChar);
extern MX_SINT16 %a_UART_Receive(MX_UINT8 nTimeout);


//This section details the static defines, prototypes and functions.

#define %a_GSM_STRING_ARRAY_SIZE	80
#define %a_MSG_ARRAY_SIZE 			80

#ifndef GSM_DEFINES
 #define GSM_DEFINES
 #define GSM_RESPONSE_OK           	1
 #define GSM_RESPONSE_ERROR        	2
 #define GSM_RESPONSE_CONNECT      	3
 #define GSM_RESPONSE_NOCARRIER    	4
 #define GSM_RESPONSE_AUDIO        	5
 #define GSM_RESPONSE_PAIR         	6
 #define GSM_RESPONSE_RING         	7
 #define GSM_EXPECTECHO

 #define GSM_CHAR_CR              	13
 #define GSM_CHAR_LF              	10

 //Timeout within the <BT_Send_Command> function (ms)
 #define GSM_SENDCMD_TIMEOUT		75

 //Timeout within the <BT_Wait_For_Response> function (ms)
 #define GSM_RESPONSE_TIMEOUT     	75

 //Timeout within the <BT_String_Receive> function (ms)
 #define GSM_STRINGRX_TIMEOUT     	150

 //Timeout within the <BT_Send_Script> function (ms)
 #define GSM_SENDSCRIPT_TIMEOUT   	75
#endif


//common global variables
MX_UINT8 %a_GSM_String_Array[%a_GSM_STRING_ARRAY_SIZE];
MX_UINT8 %a_GSM_MSG_ARRAY[%a_MSG_ARRAY_SIZE];
MX_UINT8 %a_GSM_RX_LENGTH = 0;
MX_UINT8 %a_GSM_MSG_LENGTH = 0;
MX_UINT8 %a_GSM_RX_DONE = 0;
MX_UINT8 %a_GSM_NUMBER[14];

MX_UINT8 %a_GSM_MSG_C = 0;
MX_UINT8 %a_GSM_RING_C = 0;

/*DefinesCode_End*/

/*InitialisationCode_Start*/

	%a_UART_Init();		//Call initialise function
	%a_GSM_NUMBER[0] = 0;

/*InitialisationCode_End*/

/*InterruptCode_Start*/

#if (%b == 1)
	MX_UART_INT_CH1_FUNC
#endif
#if (%b == 2)
	MX_UART_INT_CH2_FUNC
#endif
#if (%b == 3)
	MX_UART_INT_CH3_FUNC
#endif
#if (%b == 4)
	MX_UART_INT_CH4_FUNC
#endif

	MX_UINT8 %a_gsm_rx_tst = 0;

	//Test for GSM UART receive interrupt
	#if (%b == 1)
		%a_gsm_rx_tst = MX_UART_INT_CH1_DET;
	#endif
	#if (%b == 2)
		%a_gsm_rx_tst = MX_UART_INT_CH2_DET;
	#endif
	#if (%b == 3)
		%a_gsm_rx_tst = MX_UART_INT_CH2_DET;
	#endif
	#if (%b == 4)
		%a_gsm_rx_tst = MX_UART_INT_CH2_DET;
	#endif

	if (%a_gsm_rx_tst)										// Interrupt detected?
	{
		MX_UINT8 %a_gsm_in;

		// Disable UART receive interrupts
		#if (%b == 1)
			MX_UART_INT_CH1_DIS;
		#endif
		#if (%b == 2)
			MX_UART_INT_CH2_DIS;
		#endif
		#if (%b == 3)
			MX_UART_INT_CH3_DIS;
		#endif
		#if (%b == 4)
			MX_UART_INT_CH4_DIS;
		#endif

		%a_gsm_in = %a_UART_Receive(1);
		if ((%a_gsm_in == 'R') && (%a_GSM_RING_C == 0))
		{
			%a_GSM_RING_C = 1;
		}
		else if (%a_GSM_RING_C > 0)
		{
			if ((%a_gsm_in == 'I') && (%a_GSM_RING_C == 1))
			{
				%a_GSM_RING_C = 2;
			}
			else
			{
				if ((%a_gsm_in == 'N') && (%a_GSM_RING_C == 2))
				{
					%a_GSM_RING_C = 3;
				}
				else
				{
					if ((%a_gsm_in == 'G') && (%a_GSM_RING_C == 3))
					{
						%a_GSM_RING_C = 4;
					}
					else
					{
						if (%a_GSM_RING_C != 4)
							%a_GSM_RING_C = 0;
					}
				}
			}
		}


		if ((%a_gsm_in == '+') && (%a_GSM_MSG_C == 0))
		{
			%a_GSM_MSG_C = 1;
		}
		else if (%a_GSM_MSG_C > 0)
		{
			if ((%a_gsm_in == 'C') && (%a_GSM_MSG_C == 1))
			{
				%a_GSM_MSG_C = 2;
			}
			else
			{
				if ((%a_gsm_in == 'M') && (%a_GSM_MSG_C == 2))
				{
					%a_GSM_MSG_C = 3;
				}
				else
				{
					if ((%a_gsm_in == 'T') && (%a_GSM_MSG_C == 3))
					{
						%a_GSM_MSG_C = 7;
						%a_GSM_MSG_LENGTH = 0;
					}
					else
					{
						if ((%a_GSM_MSG_C < 4))
						{
							%a_GSM_MSG_C = 0;
						}
						else
						{
							if ((%a_GSM_MSG_C >= 7))
							{
								if ((%a_GSM_MSG_C >= 10) && (%a_gsm_in != '"'))			//Pull out phone number - 34 = '"'
								{
									%a_GSM_NUMBER[%a_GSM_MSG_C - 10] = %a_gsm_in;
									%a_GSM_MSG_C = %a_GSM_MSG_C + 1;
								}
								else
								{
									if ((%a_GSM_MSG_C >= 10) && (%a_gsm_in == '"'))		//End of phone number - 34 = '"'
									{
										%a_GSM_NUMBER[%a_GSM_MSG_C - 10] = 0;
										%a_GSM_MSG_C = 5;
									}
									else
									{
										%a_GSM_MSG_C = %a_GSM_MSG_C + 1;
									}
								}
							}
							else
							{
								if ((%a_GSM_MSG_C == 5))			//Wait for end of Msg Header
								{
									if (%a_gsm_in == 0x0A)
										%a_GSM_MSG_C = 6;
								}
								else
								{
									if ((%a_GSM_MSG_C == 6))			//Collect Msg Text
									{
										if (%a_gsm_in == 0x0A)
										{
											if (%a_GSM_MSG_LENGTH < %a_MSG_ARRAY_SIZE)
											{
												%a_GSM_MSG_ARRAY[%a_GSM_MSG_LENGTH] = 0;
											}
											%a_GSM_MSG_C = 4;
										}
										else if (%a_GSM_MSG_LENGTH < %a_MSG_ARRAY_SIZE)
										{
											%a_GSM_MSG_ARRAY[%a_GSM_MSG_LENGTH] = %a_gsm_in;
											%a_GSM_MSG_LENGTH = %a_GSM_MSG_LENGTH + 1;
										}

									}

								}
							}
						}
					}
				}
			}
		}

		if (%a_GSM_RX_DONE == 0)
		{
			if (%a_GSM_RX_LENGTH < %a_GSM_STRING_ARRAY_SIZE)
			{
				if (%a_gsm_in == 0x0A)
				{
					%a_GSM_RX_DONE = 1;
					%a_GSM_String_Array[%a_GSM_RX_LENGTH] = 0;
				}
				else
				{
					%a_GSM_String_Array[%a_GSM_RX_LENGTH] = %a_gsm_in;
					%a_GSM_RX_LENGTH = %a_GSM_RX_LENGTH + 1;
				}
			}
		}

		// Enable UART receive interrupts & clear interrupt
		#if (%b == 1)
			MX_UART_INT_CH1_EN;
			MX_UART_INT_CH1_CLR;
		#endif
		#if (%b == 2)
			MX_UART_INT_CH2_EN;
			MX_UART_INT_CH2_CLR;
		#endif
		#if (%b == 3)
			MX_UART_INT_CH3_EN;
			MX_UART_INT_CH3_CLR;
		#endif
		#if (%b == 4)
			MX_UART_INT_CH4_EN;
			MX_UART_INT_CH4_CLR;
		#endif
	}

MX_UART_INT_FUNC_END

/*InterruptCode_End*/

/********************************************************************
 * FUNCTIONS
 ********************************************************************/


MX_UINT8 Initialize()
{
/*Macro_Initialize_Start*/

//0=ok
//255=no reply
//254=Command not accepted

	char Retval;

	StringReceive();

	Retval = SendCommand("AT+CMGF=1",9,1);		//Default: Format SMS message as text

	if (Retval)
		return Retval;

	if ((%a_GSM_String_Array[0] != 'O') || (%a_GSM_String_Array[1] != 'K'))
	{
		return 254;
	}

	Wdt_Delay_Ms(100);

	Retval = SendCommand("AT+CNMI=1,2,0,0,0",17,1);		//Default: SMS is not stored

	if (Retval)
		return Retval;

	if ((%a_GSM_String_Array[0] != 'O') || (%a_GSM_String_Array[1] != 'K'))
	{
		return 254;
	}

	Wdt_Delay_Ms(100);

	return Retval;

/*Macro_Initialize_End*/
}



MX_UINT8 CheckNetworkStatus()
{
/*Macro_CheckNetworkStatus_Start*/

//0 = not connected
//1 = connected
//255 = no reply

	char retval = 0;

	SendCommand("AT+CREG?",8,1);		//Check network status

	if ( StringReceive() == 0)
		return 255;

	if ( %a_GSM_String_Array[7] == '0' && %a_GSM_String_Array[9] == '1' )
		retval = 1;

	StringReceive();
	StringReceive();
	Wdt_Delay_Ms(100);

	return retval;

/*Macro_CheckNetworkStatus_End*/
}


void SendCommand(MX_STRING command, MX_UINT8 MSZ_command, MX_UINT8 bSendCR)
{
/*Macro_SendCommand_Start*/

//0=ok
//255=no reply

    MX_UINT8 idx = 0;
    MX_UINT8 dummy;

	while ( StringReceive());			//Purge any unwanted data

    if (MSZ_command > 0)
    {
        while (idx < MSZ_command)
        {
            %a_UART_Send (command[idx]);
            idx++;
        }
        if (bSendCR != 0)
        {
            %a_UART_Send (0x0D);            //send the terminating CR
            MSZ_command = MSZ_command + 1;
        }
	}

	#ifdef GSM_EXPECTECHO
		StringReceive();					//Collect Echo
	#endif

	if ( StringReceive() == 0)
	{
		return 255;
	}

	return 0;

/*Macro_SendCommand_End*/
}



MX_UINT8 StringReceive(void)
{
/*Macro_StringReceive_Start*/

 //0=timeout
 //1=valid data

 	MX_UINT8 tout = 0;
 	MX_UINT8 len = %a_GSM_RX_LENGTH;

 	%a_GSM_RX_DONE = 0;
 	%a_GSM_RX_LENGTH = 0;

     while ((%a_GSM_RX_DONE == 0) && (tout < 255))
     {
 		tout = tout + 1;
 		if (len != %a_GSM_RX_LENGTH)
 		{
 			tout = 0;
 			len = %a_GSM_RX_LENGTH;
 		}
 		delay_10us(50);
     }

     if (tout == 255)
     {
 		%a_GSM_RX_DONE = 0;
 		%a_GSM_RX_LENGTH = 0;
     	return 0;
 	}
    return 1;

/*Macro_StringReceive_End*/
}



MX_UINT8 SendScript(MX_UINT8 idx)
{
/*Macro_SendScript_Start*/

    MX_UINT8 i = 0;
    MX_UINT8 ch = 255;
    MX_UINT8 retval = 0;

    if ((idx < 1) || (idx > %a_GSM_SCRIPT_COUNT))
    {
        return (0xFF);        //error - index out of range
    }
    while (ch > 0)
    {
        switch (idx)
        {
          #if (%a_GSM_SCRIPT_COUNT >= 1)
            case 1:
                ch = %a_GSM_SCRIPT_1[i];
                break;
          #endif

          #if (%a_GSM_SCRIPT_COUNT >= 2)
            case 2:
                ch = %a_GSM_SCRIPT_2[i];
                break;
          #endif

          #if (%a_GSM_SCRIPT_COUNT >= 3)
            case 3:
                ch = %a_GSM_SCRIPT_3[i];
                break;
          #endif

          #if (%a_GSM_SCRIPT_COUNT == 4)
            case 4:
                ch = %a_GSM_SCRIPT_4[i];
                break;
          #endif

            default:
                return (0xFF);    //should not get here
        }

        if (ch > 0)
        {
            //send the character
            %a_UART_Send (ch);

            //if we sent a CR, wait for the response
            if (ch == 13)
            {
				#ifdef GSM_EXPECTECHO
					StringReceive();					//Collect Echo
				#endif

				retval = StringReceive();				//Collect Reply

				while ( StringReceive());				//Make sure we have received everything
            }
        }
        i++;
    }
    return (retval);

/*Macro_SendScript_End*/
}


MX_UINT8 CheckForIncoming ()
{
/*Macro_CheckForIncoming_Start*/

	StringReceive();					//Read any incoming messages

	if (%a_GSM_MSG_C == 4)					//Text has been received
	{
		%a_GSM_MSG_C = 0;
		return 1;						//Text Received
	}
	else if (%a_GSM_RING_C == 4)
	{
		%a_GSM_RING_C = 0;
		return 2;						//Phone Ringing
	}
	return 0;

/*Macro_CheckForIncoming_End*/
}


MX_UINT8 DialNumber (MX_STRING number)
{
/*Macro_DialNumber_Start*/

	MX_UINT8 idx;

	%a_UART_Send ('A');
	%a_UART_Send ('T');
	%a_UART_Send ('D');

	for (idx=0; idx<MSZ_number; idx++)
	{
		%a_UART_Send (number[idx]);
	}

	%a_UART_Send (';');
	%a_UART_Send (GSM_CHAR_CR);

	#ifdef GSM_EXPECTECHO
		StringReceive();					//Collect Echo
	#endif

	StringReceive();

	if ((%a_GSM_String_Array[0] == 'O') && (%a_GSM_String_Array[1] == 'K'))
		return 0;

	return 255;

/*Macro_DialNumber_End*/
}



MX_UINT8 HangUpCall ()
{
/*Macro_HangUpCall_Start*/

	%a_UART_Send ('A');
	%a_UART_Send ('T');
	%a_UART_Send ('H');
	%a_UART_Send (GSM_CHAR_CR);

	#ifdef GSM_EXPECTECHO
		StringReceive();					//Collect Echo
	#endif

	StringReceive();

	if ((%a_GSM_String_Array[0] == 'O') && (%a_GSM_String_Array[1] == 'K'))
		return 0;

	return 255;

/*Macro_HangUpCall_End*/
}



MX_UINT8 AnswerIncomingCall()
{
/*Macro_AnswerIncomingCall_Start*/

	%a_UART_Send ('A');
	%a_UART_Send ('T');
	%a_UART_Send ('A');
	%a_UART_Send (GSM_CHAR_CR);

	#ifdef GSM_EXPECTECHO
		StringReceive();					//Collect Echo
	#endif

	StringReceive();

	if ((%a_GSM_String_Array[0] == 'O') && (%a_GSM_String_Array[1] == 'K'))
		return 0;

	return 255;

/*Macro_AnswerIncomingCall_End*/
}



MX_UINT8 SendTextMessage (MX_STRING number, MX_STRING message)
{
/*Macro_SendTextMessage_Start*/

	MX_UINT8 idx;

	%a_UART_Send ('A');
	%a_UART_Send ('T');
	%a_UART_Send ('+');
	%a_UART_Send ('C');
	%a_UART_Send ('S');
	%a_UART_Send ('M');
	%a_UART_Send ('P');
	%a_UART_Send ('=');
	%a_UART_Send ('1');
	%a_UART_Send ('7');
	%a_UART_Send (',');
	%a_UART_Send ('1');
	%a_UART_Send ('6');
	%a_UART_Send ('7');
	%a_UART_Send (',');
	%a_UART_Send ('0');
	%a_UART_Send (',');
	%a_UART_Send ('0');
	%a_UART_Send (GSM_CHAR_CR);

	#ifdef GSM_EXPECTECHO
		StringReceive();					//Collect Echo
	#endif

	StringReceive();
	Wdt_Delay_Ms(100);

	%a_UART_Send ('A');
	%a_UART_Send ('T');
	%a_UART_Send ('+');
	%a_UART_Send ('C');
	%a_UART_Send ('S');
	%a_UART_Send ('D');
	%a_UART_Send ('H');
	%a_UART_Send ('=');
	%a_UART_Send ('1');
	%a_UART_Send (GSM_CHAR_CR);

	#ifdef GSM_EXPECTECHO
		StringReceive();					//Collect Echo
	#endif

	StringReceive();
	Wdt_Delay_Ms(100);

	%a_UART_Send ('A');
	%a_UART_Send ('T');
	%a_UART_Send ('+');
	%a_UART_Send ('C');
	%a_UART_Send ('M');
	%a_UART_Send ('G');
	%a_UART_Send ('S');
	%a_UART_Send ('=');
	%a_UART_Send ('"');

	for (idx=0; idx<MSZ_number; idx++)
	{
		if (number[idx] == 0)
			break;

		%a_UART_Send (number[idx]);
	}

	%a_UART_Send ('"');
	%a_UART_Send (GSM_CHAR_CR);

	#ifdef GSM_EXPECTECHO
		StringReceive();					//Collect Echo
	#endif

	StringReceive();
	Wdt_Delay_Ms(100);

	for (idx=0; idx<MSZ_message; idx++)
	{
		if (message[idx] == 0)
			break;

		%a_UART_Send (message[idx]);
	}

	%a_UART_Send (26);

	#ifdef GSM_EXPECTECHO
		StringReceive();					//Collect Echo
	#endif

	StringReceive();
	Wdt_Delay_Ms(100);

	if ((%a_GSM_String_Array[0] == 'O') && (%a_GSM_String_Array[1] == 'K'))
		return 0;

	return 255;

/*Macro_SendTextMessage_End*/
}



char DeleteAllMessages()
{
/*Macro_DeleteAllMessages_Start*/

	SendCommand("AT+CMGD=1,4",11,1);

	StringReceive();

	if ((%a_GSM_String_Array[0] == 'O') && (%a_GSM_String_Array[1] == 'K'))
		return 0;

	return 255;

/*Macro_DeleteAllMessages_End*/
}



void ReadString (char* FCR_RETVAL, char FCR_RETVAL_SIZE)
{
/*Macro_ReadString_Start*/

	char idx = 0;

	while ((idx < FCR_RETVAL_SIZE) && (%a_GSM_String_Array[idx] > 0))
	{
		FCR_RETVAL[idx] = %a_GSM_String_Array[idx];
		idx = idx + 1;
	}
	if (idx < FCR_RETVAL_SIZE)
		FCR_RETVAL[idx] = 0;			//String termination

/*Macro_ReadString_End*/
}




void GetTextNumber (MX_STRING FCR_RETVAL, MX_UINT8 FCR_RETVAL_SIZE)
{
/*Macro_GetTextNumber_Start*/

	MX_UINT8 idx = 0;

	while (idx < FCR_RETVAL_SIZE && %a_GSM_NUMBER[idx] > 0 )
	{
		FCR_RETVAL[idx] = %a_GSM_NUMBER[idx];
		idx = idx + 1;
	}
	if (idx < FCR_RETVAL_SIZE)
		FCR_RETVAL[idx] = 0;			//String termination

/*Macro_GetTextNumber_End*/
}



void GetTextString (char* FCR_RETVAL, char FCR_RETVAL_SIZE)
{
/*Macro_GetTextString_Start*/

	char idx = 0;

	while ((idx < %a_GSM_MSG_LENGTH) && (%a_GSM_MSG_ARRAY[idx] > 0))
	{
		FCR_RETVAL[idx] = %a_GSM_MSG_ARRAY[idx];
		idx = idx + 1;
	}
	if (idx < FCR_RETVAL_SIZE)
		FCR_RETVAL[idx] = 0;			//String termination

	DeleteAllMessages();

/*Macro_GetTextString_End*/
}


