/*********************************************************************
 *                    Flowcode MIDI Component Code
 *
 * File: MIDI.c
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 100608 | BR | Created
 * 240308 | BR | Moved private functions from AddDefines section
 * 130509 | BR | Added % definitions to DefinesCode section
 * 230311 | BR | Changed code to work for hitech and boostc without ifdefs
 * 071011 | BR | Converted to Universal MIDI Component Code File
 * 011211 | BR | Converted to new dynamic defines mechanism
 * 011211 | BR | Updated FC_CAL_Bit functions
 ********************************************************************/

/*********************************************************************

 Return & parameter types:
   void
   MX_UINT8
   MX_SINT16
   MX_STRING

 Pin directions:
   0 = OUTPUT
   2 = INPUT
   3 = BIDIRECTIONAL

**********************************************************************

[Settings]
CLSID={655D06CC-07C3-4A46-88C9-12DA8A2D56D3}
IsAnalogue=0
MultipleAllowed=1
Description=MIDI Flowcode Component

**********************************************************************

[MX_MIDI_PORT]
DesiredMX_MIDI_PORT=0
MX_MIDI_PORTMandatory=0
SameMX_MIDI_PORTAsID=0

[Pins]
Count=2
1=MIDI OUT
2=MIDI IN

[PinPORT]
# PORTA = 0
# PORTB = 1
# PORTC = 2
# PORTD = 3
# PORTE = 4
1=0
2=0

[PinDesiredBit]
1=0
2=1

[PinDirection]
# DIR_INPUT = 2
# DIR_OUTPUT = 0
# DIR_BIDIRECTIONAL = 3
1=0
2=2

[PinMustUsePORT]
# MX_UINT8
1=1
2=1

[SamePORTAsPinX]
# -1 = any pin
# otherwise, the pin number
1=-1
2=0

[PinMustUsePin]
# MX_UINT8
1=1
2=1

[PinValue]
# MX_UINT8 (0 = off, 1 = on)
1=0
2=0

**********************************************************************

[MacroNames]
Count=3
1=SendMIDI
2=ReceiveMIDI
3=ReadData

[MacroReturns]
1=void
2=MX_UINT8
3=MX_UINT8

[MacroIsPrivate]
1=0
2=0
3=0


[MacroParameters_SendMIDI]
Count=3
1=cStatus
2=cData1
3=cData2

[MacroParamTypes_SendMIDI]
1=MX_UINT8
2=MX_UINT8
3=MX_UINT8


[MacroParameters_ReceiveMIDI]
Count=1
1=cTimeout

[MacroParamTypes_ReceiveMIDI]
1=MX_UINT8


[MacroParameters_ReadData]
Count=1
1=cIdx

[MacroParamTypes_ReadData]
1=MX_UINT8

 ********************************************************************/


/********************************************************************
 * ADDITIONAL CODE
 ********************************************************************/

/*DefinesCode_Start*/

/**** Macro Substitutions ****
a = Unique Reference
b = Uart Channel		(0-software / 1-4 hardware)
c = tx pin
d = tx port
e = rx pin
f = rx port
******************************/

//Definitions for RS232 UART slot allocation
#ifndef	MX_UART_REF1
	#define MX_UART_REF1
	#define %a_MX_UART_UREF			1
	#define MX_UART_CHANNEL_1		%b
	#define MX_UART_TX_PIN_1		%c
	#define MX_UART_TX_PORT_1		port%d
	#define MX_UART_TX_TRIS_1		tris%d
	#define MX_UART_RX_PIN_1		%e
	#define MX_UART_RX_PORT_1		port%f
	#define MX_UART_RX_TRIS_1		tris%f
	#define MX_UART_FLOWEN_1		0
	#define MX_UART_CTS_PIN_1		0
	#define MX_UART_CTS_PORT_1		0
	#define MX_UART_RTS_PIN_1		0
	#define MX_UART_RTS_PORT_1		0
	#define MX_UART_BAUD_1			31250		//MIDI has set baud of 31250bps
	#define MX_UART_DBITS_1			8
	#define MX_UART_RETURN_1		0
	#define MX_UART_ECHO_1			0
	#define MX_UART_INT_1			0
#else
 #ifndef	MX_UART_REF2
	#define MX_UART_REF2
	#define %a_MX_UART_UREF			2
	#define MX_UART_CHANNEL_2		%b
	#define MX_UART_TX_PIN_2		%c
	#define MX_UART_TX_PORT_2		port%d
	#define MX_UART_TX_TRIS_2		tris%d
	#define MX_UART_RX_PIN_2		%e
	#define MX_UART_RX_PORT_2		port%f
	#define MX_UART_RX_TRIS_2		tris%f
	#define MX_UART_FLOWEN_2		0
	#define MX_UART_CTS_PIN_2		0
	#define MX_UART_CTS_PORT_2		0
	#define MX_UART_RTS_PIN_2		0
	#define MX_UART_RTS_PORT_2		0
	#define MX_UART_BAUD_2			31250		//MIDI has set baud of 31250bps
	#define MX_UART_DBITS_2			8
	#define MX_UART_RETURN_2		0
	#define MX_UART_ECHO_2			0
	#define MX_UART_INT_2			0
 #else
  #ifndef	MX_UART_REF3
	#define MX_UART_REF3
	#define %a_MX_UART_UREF			3
	#define MX_UART_CHANNEL_3		%b
	#define MX_UART_TX_PIN_3		%c
	#define MX_UART_TX_PORT_3		port%d
	#define MX_UART_TX_TRIS_3		tris%d
	#define MX_UART_RX_PIN_3		%e
	#define MX_UART_RX_PORT_3		port%f
	#define MX_UART_RX_TRIS_3		tris%f
	#define MX_UART_FLOWEN_3		0
	#define MX_UART_CTS_PIN_3		0
	#define MX_UART_CTS_PORT_3		0
	#define MX_UART_RTS_PIN_3		0
	#define MX_UART_RTS_PORT_3		0
	#define MX_UART_BAUD_3			31250		//MIDI has set baud of 31250bps
	#define MX_UART_DBITS_3			8
	#define MX_UART_RETURN_3		0
	#define MX_UART_ECHO_3			0
	#define MX_UART_INT_3			0
  #else
   #ifndef	MX_UART_REF4
	#define MX_UART_REF4
	#define %a_MX_UART_UREF			4
	#define MX_UART_CHANNEL_4		%b
	#define MX_UART_TX_PIN_4		%c
	#define MX_UART_TX_PORT_4		port%d
	#define MX_UART_TX_TRIS_4		tris%d
	#define MX_UART_RX_PIN_4		%e
	#define MX_UART_RX_PORT_4		port%f
	#define MX_UART_RX_TRIS_4		tris%f
	#define MX_UART_FLOWEN_4		0
	#define MX_UART_CTS_PIN_4		0
	#define MX_UART_CTS_PORT_4		0
	#define MX_UART_RTS_PIN_4		0
	#define MX_UART_RTS_PORT_4		0
	#define MX_UART_BAUD_4			31250		//MIDI has set baud of 31250bps
	#define MX_UART_DBITS_4			8
	#define MX_UART_RETURN_4		0
	#define MX_UART_ECHO_4			0
	#define MX_UART_INT_4			0
   #endif
  #endif
 #endif
#endif

#define %a_UART_Init		CAL_APPEND(FC_CAL_UART_Init_, %a_MX_UART_UREF)
#define %a_UART_Send		CAL_APPEND(FC_CAL_UART_Send_, %a_MX_UART_UREF)
#define %a_UART_Receive		CAL_APPEND(FC_CAL_UART_Receive_, %a_MX_UART_UREF)

extern void %a_UART_Init();
extern void %a_UART_Send(MX_UINT16 nChar);
extern MX_SINT16 %a_UART_Receive(MX_UINT8 nTimeout);

//Global Variables
MX_UINT8 %a_Received_MIDI[4];

/*DefinesCode_End*/

/*InitialisationCode_Start*/

	%a_UART_Init();		//Call initialise function

/*InitialisationCode_End*/

/*InterruptCode_Start*/
/*InterruptCode_End*/

/********************************************************************
 * FUNCTIONS
 ********************************************************************/


void SendMIDI(MX_UINT8 cStatus, MX_UINT8 cData1, MX_UINT8 cData2)
{
/*Macro_SendMIDI_Start*/

//***************************************************************************
//Outputs a MIDI signal in serial format with a single status byte
//and one or two optional data bytes. If data bytes are not needed then send
//255 which ends the transmission routine
//***************************************************************************/

  	MX_UINT8 cByte, current_data;

	for(cByte = 0; cByte < 3; cByte++)  				//Transmit up to 3 data bytes
  	{
    	switch(cByte)
    	{
      		case 0:
      			current_data = cStatus;  				//Load current tx Byte
      			break;

      		case 1:
      			current_data = cData1;
      			break;

      		case 2:
      			current_data = cData2;
      			break;
    	}

    	if (cByte != 0 && current_data >= 128)
    		return;   									//>127 = no data and ends routine

		%a_UART_Send(current_data);
	}
/*Macro_SendMIDI_End*/
}


MX_UINT8 ReceiveMIDI(MX_UINT8 cTimeout)
{
/*Macro_ReceiveMIDI_Start*/

//********************************************************************
//returns 0 for no data, or the status byte if data has been received
//cTimeout represents number of data bits to wait before returning.
//If data is recieved before timeout occurs then timeout is reset
//********************************************************************/

  	MX_UINT8 idx, current_data = 0;

	%a_Received_MIDI[0] = 0;								//Reset Status Byte
	%a_Received_MIDI[3] = 0;								//Reset Real Time Message

  	for (idx = 1; idx < 3; idx++)
  	{
		%a_Received_MIDI[idx] = 255;						//Clear old MIDI data
	}

	for (idx = 0; idx < 3; idx++)
	{
		current_data = %a_UART_Receive(cTimeout);
		if (current_data == 255)
			return 0;										//Timeout Occurred

		if (current_data >= 0xf8)							//If real time message received
			%a_Received_MIDI[3] = current_data;
		else												//Else normal message / data
			%a_Received_MIDI[idx] = current_data;
	}
	return %a_Received_MIDI[0];  							//Return status byte

/*Macro_ReceiveMIDI_End*/
}


MX_UINT8 ReadData(MX_UINT8 cIdx)
{
/*Macro_ReadData_Start*/

//************************************
//returns the data byte (cIdx = 1 - 2)
//or a real time message (cIdx = 3)
//***********************************/

	if(cIdx < 4)
  		return %a_Received_MIDI[cIdx];						//Return byte from the last incoming MIDI message
  	return 0;

/*Macro_ReadData_End*/
}
