/*********************************************************************
 *                Flowcode Wireless LAN Component Code
 *
 * File: WLAN.c
 *
 * (c) 2009 Matrix Multimedia Ltd.
 * http://www.matrixmultimedia.com
 *
 * Software License Agreement
 *
 * The software supplied herewith by Matrix Multimedia Ltd (the
 * Company) for its Flowcode graphical programming language is
 * intended and supplied to you, the Companys customer, for use
 * solely and exclusively on the Company's products. The software
 * is owned by the Company, and is protected under applicable
 * copyright laws. All rights are reserved. Any use in violation
 * of the foregoing restrictions may subject the user to criminal
 * sanctions under applicable laws, as well as to civil liability
 * for the breach of the terms and conditions of this licence.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Changelog:
 *
 *  date  | by | description
 * -------+----+-----------------------------------------------------
 * 120511 | BR | Created
 * 130511 | BR | Added Swap Out_Variables and page functions
 * 150611 | BR | Finalised component code
 * 150611 | BR | Added passback variable functionality
 * 210611 | BR | Fixed some minor bugs and allowed connect to SSID function to work
 * 270611 | BR | Added new check status function and improved code efficiency
 * 071011 | BR | Converted to Universal WLAN Component Code File
 * 111111 | ST | Added html pages and pagecount
 * 011211 | BR | Converted to new dynamic defines mechanism
 * 011211 | BR | Updated FC_CAL_Bit functions
 ********************************************************************/

/*********************************************************************

 Return & parameter types:
   void
   char
   short
   char*

 Pin directions:
   0 = OUTPUT
   2 = INPUT
   3 = BIDIRECTIONAL

**********************************************************************

[Settings]
CLSID={DA77448F-7831-4F3E-969E-6AE4EE551B6F}
IsAnalogue=0
MultipleAllowed=0
Description=Wireless LAN Flowcode Component

**********************************************************************

[Port]
DesiredPort=0
PortMandatory=0
SamePortAsID=0

[Pins]
Count=0

[PinPort]
# PORTA = 0
# PORTB = 1
# PORTC = 2
# PORTD = 3
# PORTE = 4

[PinDesiredBit]

[PinDirection]
# DIR_INPUT = 2
# DIR_OUTPUT = 0
# DIR_BIDIRECTIONAL = 3

[PinMustUsePort]
# boolean

[SamePortAsPinX]
# -1 = any pin
# otherwise, the pin number

[PinMustUsePin]
# boolean

[PinValue]
# boolean (0 = off, 1 = on)

**********************************************************************

[MacroNames]
Count=10
1=Initialise
2=Send_Command
3=Get_Reply
4=Receive_Incoming
5=Connect_To_SSID
6=Check_For_Page_Requests
7=Send_String
8=Write_Swap_Variable
9=Read_Swap_Variable
10=Read_Connection_Status

[MacroReturns]
1=char
2=char
3=char
4=char*
5=char
6=char
7=void
8=void
9=char
10=char

[MacroIsPrivate]
1=0
2=0
3=1
4=0
5=0
6=0
7=0
8=0
9=0
10=0


[MacroParameters_Initialise]
Count=0

[MacroParamTypes_Initialise]


[MacroParameters_Send_Command]
Count=1
1=String

[MacroParamTypes_Send_Command]
1=char*

[MacroParameters_Get_Reply]
Count=0

[MacroParamTypes_Get_Reply]


[MacroParameters_Receive_Incoming]
Count=0

[MacroParamTypes_Receive_Incoming]


[MacroParameters_Connect_To_SSID]
Count=2
1=SSID
2=Security_Key

[MacroParamTypes_Connect_To_SSID]
1=char*
2=char*

[MacroParameters_Check_For_Page_Requests]
Count=0

[MacroParamTypes_Check_For_Page_Requests]


[MacroParameters_Send_String]
Count=2
1=String
2=Append_CRLF

[MacroParamTypes_Send_String]
1=char*
2=char

[MacroParameters_Write_Swap_Variable]
Count=2
1=index
2=data

[MacroParamTypes_Write_Swap_Variable]
1=char
2=char

[MacroParameters_Read_Swap_Variable]
Count=1
1=index

[MacroParamTypes_Read_Swap_Variable]
1=char


[MacroParameters_Read_Connection_Status]
Count=0

[MacroParamTypes_Read_Connection_Status]


 ********************************************************************/

/********************************************************************
 * ADDITIONAL CODE
 ********************************************************************/

/*DefinesCode_Start*/

/**** Macro Substitutions ****
a = Unique Reference
b = Uart Channel		(0-software / 1-4 hardware)
c = tx pin
d = tx port
e = rx pin
f = rx port
g = WLAN Mode - 0=Accesspoint / 3=End Device
h = Broadcast SSID - Mode AP only - 0=on, 1=off
i = Wireless Channel - Mode AP only - 0=auto or 1 to 13
j = Transmission Power - 0 = Off or 1 to 16 - default 14
k = WLAN Data Rate - 1/2/5/11/24/36/48/54 in Mbps - default 48Mbps
l = DHCP 1=enabled / 0=disabled
m = Server Port
n = SSID - Name of WLAN network
o = Alias - Name of network instance
p = Page Request Timeout
q = Security Mode - 0=Open / 1=Retain settings
r = WLAN IP Addresses
s = WLAN HTML Pages
t = Trigger Port
u = Trigger Pin
v = Reset Port
w = Reset Pin
x = PageNames
y = PAGE1_HTML
za = PAGE2_HTML
zb = PAGE3_HTML
zc = PAGE4_HTML
******************************/

#define MX_WLAN_MODE	"GO%g"			//0=Accesspoint / 3=End Device
#define MX_WLAN_MODEA	%g
#define MX_WLAN_BSSID	"GH%h"			//Broadcast SSID - Mode AP only - 0=on, 1=off
#define MX_WLAN_CHAN	"GC%i"			//Wireless Channel - Mode AP only - 0=auto or 1 to 13
#define MX_WLAN_TXP		"GP%j"			//Transmision power - 0 = Off or 1 to 16 - default 14
#define MX_WLAN_RATE	"GR%k"			//1/2/5/11/24/36/48/54 in Mbps - default 48Mbps
#define MX_WLAN_DHCP	"WD%l"			//DHCP 1=enabled / 0=disabled
#define MX_WLAN_DHCPA	%l
#define MX_WLAN_PORT	"WP%m"			//Server Port
#define MX_WLAN_SSID	"GS%n"			//SSID - Name of WLAN network
#define MX_WLAN_ALIAS	"GN%o"			//Alias - Name of network instance
#define MX_WLAN_TOUT	"OI%p"			//Page Request Timeout
#define MX_WLAN_SMODE	%q				//Security Mode - 0=Open / 1=Retain settings

#define MX_WLAN_TRIG_PORT		port%t
#define MX_WLAN_TRIG_TRIS		tris%t
#define MX_WLAN_TRIG_PIN		%u
#define MX_WLAN_RESET_PORT		port%v
#define MX_WLAN_RESET_TRIS		tris%v
#define MX_WLAN_RESET_PIN		%w

%r

//Definitions for WLAN UART slot allocation
#ifndef	MX_UART_REF1
	#ifndef %a_MX_UART_UREF
		#define MX_UART_REF1
		#define %a_MX_UART_UREF			1
		#define MX_UART_CHANNEL_1		%b
		#define MX_UART_TX_PIN_1		%c
		#define MX_UART_TX_PORT_1		port%d
		#define MX_UART_TX_TRIS_1		tris%d
		#define MX_UART_RX_PIN_1		%e
		#define MX_UART_RX_PORT_1		port%f
		#define MX_UART_RX_TRIS_1		tris%f
		#define MX_UART_FLOWEN_1		0
		#define MX_UART_CTS_PIN_1		0
		#define MX_UART_CTS_PORT_1		0
		#define MX_UART_RTS_PIN_1		0
		#define MX_UART_RTS_PORT_1		0
		#define MX_UART_BAUD_1			38400
		#define MX_UART_DBITS_1			8
		#define MX_UART_RETURN_1		0
		#define MX_UART_ECHO_1			0
		#define MX_UART_INT_1			1
	#endif
#else
	#ifndef	MX_UART_REF2
		#ifndef %a_MX_UART_UREF
			#define MX_UART_REF2
			#define %a_MX_UART_UREF			2
			#define MX_UART_CHANNEL_2		%b
			#define MX_UART_TX_PIN_2		%c
			#define MX_UART_TX_PORT_2		port%d
			#define MX_UART_TX_TRIS_2		tris%d
			#define MX_UART_RX_PIN_2		%e
			#define MX_UART_RX_PORT_2		port%f
			#define MX_UART_RX_TRIS_2		tris%f
			#define MX_UART_FLOWEN_2		0
			#define MX_UART_CTS_PIN_2		0
			#define MX_UART_CTS_PORT_2		0
			#define MX_UART_RTS_PIN_2		0
			#define MX_UART_RTS_PORT_2		0
			#define MX_UART_BAUD_2			38400
			#define MX_UART_DBITS_2			8
			#define MX_UART_RETURN_2		0
			#define MX_UART_ECHO_2			0
			#define MX_UART_INT_2			1
		#endif
	#else
		#ifndef	MX_UART_REF3
			#ifndef %a_MX_UART_UREF
				#define MX_UART_REF3
				#define %a_MX_UART_UREF			3
				#define MX_UART_CHANNEL_3		%b
				#define MX_UART_TX_PIN_3		%c
				#define MX_UART_TX_PORT_3		port%d
				#define MX_UART_TX_TRIS_3		tris%d
				#define MX_UART_RX_PIN_3		%e
				#define MX_UART_RX_PORT_3		port%f
				#define MX_UART_RX_TRIS_3		tris%f
				#define MX_UART_FLOWEN_3		0
				#define MX_UART_CTS_PIN_3		0
				#define MX_UART_CTS_PORT_3		0
				#define MX_UART_RTS_PIN_3		0
				#define MX_UART_RTS_PORT_3		0
				#define MX_UART_BAUD_3			38400
				#define MX_UART_DBITS_3			8
				#define MX_UART_RETURN_3		0
				#define MX_UART_ECHO_3			0
				#define MX_UART_INT_3			1
			#endif
		#else
			#ifndef	MX_UART_REF4
				#ifndef %a_MX_UART_UREF
					#define MX_UART_REF4
					#define %a_MX_UART_UREF			4
					#define MX_UART_CHANNEL_4		%b
					#define MX_UART_TX_PIN_4		%c
					#define MX_UART_TX_PORT_4		port%d
					#define MX_UART_TX_TRIS_4		tris%d
					#define MX_UART_RX_PIN_4		%e
					#define MX_UART_RX_PORT_4		port%f
					#define MX_UART_RX_TRIS_4		tris%f
					#define MX_UART_FLOWEN_4		0
					#define MX_UART_CTS_PIN_4		0
					#define MX_UART_CTS_PORT_4		0
					#define MX_UART_RTS_PIN_4		0
					#define MX_UART_RTS_PORT_4		0
					#define MX_UART_BAUD_4			38400
					#define MX_UART_DBITS_4			8
					#define MX_UART_RETURN_4		0
					#define MX_UART_ECHO_4			0
					#define MX_UART_INT_4			1
				#endif
			#endif
		#endif
	#endif
#endif

#define %a_UART_Init		CAL_APPEND(FC_CAL_UART_Init_, %a_MX_UART_UREF)
#define %a_UART_Send		CAL_APPEND(FC_CAL_UART_Send_, %a_MX_UART_UREF)
#define %a_UART_Receive		CAL_APPEND(FC_CAL_UART_Receive_, %a_MX_UART_UREF)

extern void %a_UART_Init();
extern void %a_UART_Send(MX_UINT16 nChar);
extern MX_SINT16 %a_UART_Receive(MX_UINT8 nTimeout);

//the following defines the number of HTML pages and the names of these pages
#define MX_WLAN_MAX_PAGES%s

//Defines to automatically code the appropriate pages...
#ifdef MX_WLAN_MAX_PAGES1
  #define MX_WLAN_MAX_PAGES    1
  #define MX_WLAN_USE_PAGE1
#endif
#ifdef MX_WLAN_MAX_PAGES2
  #define MX_WLAN_MAX_PAGES    2
  #define MX_WLAN_USE_PAGE1
  #define MX_WLAN_USE_PAGE2
#endif
#ifdef MX_WLAN_MAX_PAGES3
  #define MX_WLAN_MAX_PAGES    3
  #define MX_WLAN_USE_PAGE1
  #define MX_WLAN_USE_PAGE2
  #define MX_WLAN_USE_PAGE3
#endif
#ifdef MX_WLAN_MAX_PAGES4
  #define MX_WLAN_MAX_PAGES    4
  #define MX_WLAN_USE_PAGE1
  #define MX_WLAN_USE_PAGE2
  #define MX_WLAN_USE_PAGE3
  #define MX_WLAN_USE_PAGE4
#endif

%x

//Static defines
#define MX_WLAN_CMD_OK			0
#define MX_WLAN_CMD_FAIL_ERR	1
#define MX_WLAN_CMD_STX_ERR		2
#define MX_WLAN_CMD_CMD_ERR		3
#define MX_WLAN_CMD_PARAM_ERR	4
#define MX_WLAN_CMD_ETX_ERR		5
#define MX_WLAN_CMD_SERIAL		6
#define MX_WLAN_CMD_TOUT		255

#define MX_WLAN_RX_BUFF			30			//Receive buffer Length

//global Out_Variables
char in_buffer[MX_WLAN_RX_BUFF];			//Receive buffer
char in_length = 0;
char cmd_state = 0;

#define NUM_VARS				8			//Substitution Out_Variables
char Out_Variables[NUM_VARS] = {0,0,0,0,0,0,0,0};
char In_Variables[NUM_VARS] = {0,0,0,0,0,0,0,0};


void FCD_WLAN0_Send_String(char* String, char MSZ_String, char Append_CRLF);

#ifdef MX_WLAN_USE_PAGE1
	void SendHTMLPage1()
	{
		%y
	}
#endif

#ifdef MX_WLAN_USE_PAGE2
	void SendHTMLPage2()
	{
		%za
	}
#endif

#ifdef MX_WLAN_USE_PAGE3
	void SendHTMLPage3()
	{
		%zb
	}
#endif

#ifdef MX_WLAN_USE_PAGE4
	void SendHTMLPage4()
	{
		%zc
	}
#endif

/*DefinesCode_End*/

/*InitialisationCode_Start*/

	%a_UART_Init();

/*InitialisationCode_End*/

/*InterruptCode_Start*/

MX_UINT8 %a_wlan_in;

#if (%b == 1)
	MX_UART_INT_CH1_FUNC
#endif
#if (%b == 2)
	MX_UART_INT_CH2_FUNC
#endif
#if (%b == 3)
	MX_UART_INT_CH3_FUNC
#endif
#if (%b == 4)
	MX_UART_INT_CH4_FUNC
#endif

	MX_UINT8 %a_wlan_rx_tst = 0;

	//Test for GPS UART receive interrupt
	#if (%b == 1)
		%a_wlan_rx_tst = MX_UART_INT_CH1_DET;
	#endif
	#if (%b == 2)
		%a_wlan_rx_tst = MX_UART_INT_CH2_DET;
	#endif
	#if (%b == 3)
		%a_wlan_rx_tst = MX_UART_INT_CH2_DET;
	#endif
	#if (%b == 4)
		%a_wlan_rx_tst = MX_UART_INT_CH2_DET;
	#endif

	if (%a_wlan_rx_tst)										// Interrupt detected?
	{
		// Disable UART receive interrupts
		#if (%b == 1)
			MX_UART_INT_CH1_DIS;
		#endif
		#if (%b == 2)
			MX_UART_INT_CH2_DIS;
		#endif
		#if (%b == 3)
			MX_UART_INT_CH3_DIS;
		#endif
		#if (%b == 4)
			MX_UART_INT_CH4_DIS;
		#endif

		%a_wlan_in = %a_UART_Receive(1);

		if (cmd_state == 1)									//Command Reply
		{
			if (in_length < MX_WLAN_RX_BUFF)
			{
				in_buffer[in_length] = %a_wlan_in;
				in_length = in_length + 1;
			}

			if (%a_wlan_in == '>')
				cmd_state = 0;
		}
		else if (cmd_state == 0)							//unsolicited msg
		{
			if (%a_wlan_in == 'G')
				cmd_state = 2;
		}
		else if (cmd_state == 2)
		{
			if (%a_wlan_in == 'E')
				cmd_state = 3;
			else
				cmd_state = 0;
		}
		else if (cmd_state == 3)
		{
			if (%a_wlan_in == 'T')
				cmd_state = 4;
			else
				cmd_state = 0;
		}
		else if (cmd_state == 4)
		{
			if (%a_wlan_in == ' ')
			{
				cmd_state = 5;
				in_length = 0;						//If we get here then "GET " command has been received
			}
			else
				cmd_state = 0;
		}
		else if (cmd_state == 5)
		{
			if (%a_wlan_in == ' ')
			{
				in_buffer[in_length] = 0;			//Add termination
				cmd_state = 0;
			}
			else
			{
				if (in_length < MX_WLAN_RX_BUFF)
				{
					in_buffer[in_length] = %a_wlan_in;		//Store page requested
					in_length = in_length + 1;
				}
			}
		}

		// Enable UART receive interrupts & clear interrupt
		#if (%b == 1)
			MX_UART_INT_CH1_EN;
			MX_UART_INT_CH1_CLR;
		#endif
		#if (%b == 2)
			MX_UART_INT_CH2_EN;
			MX_UART_INT_CH2_CLR;
		#endif
		#if (%b == 3)
			MX_UART_INT_CH3_EN;
			MX_UART_INT_CH3_CLR;
		#endif
		#if (%b == 4)
			MX_UART_INT_CH4_EN;
			MX_UART_INT_CH4_CLR;
		#endif
	}

MX_UART_INT_FUNC_END

/*InterruptCode_End*/

/********************************************************************
 * FUNCTIONS
 ********************************************************************/

//public function implementations


char Initialise(void)
{
/*Macro_Initialise_Start*/

	char retval = 255;

	FC_CAL_Bit_High_DDR(MX_WLAN_TRIG_PORT, MX_WLAN_TRIG_TRIS, MX_WLAN_TRIG_PIN);		//Output 0 to get module into serial command mode
	FC_CAL_Bit_Low_DDR(MX_WLAN_RESET_PORT, MX_WLAN_RESET_TRIS, MX_WLAN_RESET_PIN);		//Reset Off
	delay_us(10);
	FC_CAL_Bit_High(MX_WLAN_RESET_PORT, MX_WLAN_RESET_PIN);			//Reset Module

	delay_ms(200);

	//Uncomment this line if module locks up and no longer gets through Init
	//*******************************************************************************//
	//delay_s(30);										//Perform Factory Reset
	//*******************************************************************************//

	FC_CAL_Bit_Low(MX_WLAN_RESET_PORT, MX_WLAN_RESET_PIN);			//Reset Off
	delay_s(20);

	retval = Send_Command("OI0", 3);					//Send command and check for response

	if (retval)											//Have we already waited 1 min?
		return retval;

	Send_Command(MX_WLAN_MODE, 3);						//Operation mode

	#if (MX_WLAN_MODEA == 0) || (MX_WLAN_DHCPA == 0) 	//AP MODE or DHCP Server Off?
		Send_Command(MX_WLAN_IP, 17);					//IP
		Send_Command(MX_WLAN_MSK, 17);					//Subnet
		Send_Command(MX_WLAN_GIP, 17);					//Gateway
	#endif

	#if MX_WLAN_MODEA == 0								//AP MODE ONLY
		#if MX_WLAN_DHCPA == 1							//DHCP Server On?
			Send_Command(MX_WLAN_DCP, 33);				//DHCP Pool IP
		#endif

		Send_Command(MX_WLAN_SSID, 34);					//WLAN SSID
		Send_Command(MX_WLAN_BSSID, 3);					//Broadcast SSID
		Send_Command(MX_WLAN_CHAN, 4);					//Wireless Channel
	#endif

	Send_Command(MX_WLAN_ALIAS, 34);					//WLAN Alias

	delay_ms(10);

	Send_Command(MX_WLAN_TXP, 4);						//Transmission Power
	Send_Command(MX_WLAN_RATE, 4);						//Data Rate
	Send_Command(MX_WLAN_DHCP, 3);						//DHCP Server
	Send_Command(MX_WLAN_TOUT, 4);						//Inactivity Time
	Send_Command(MX_WLAN_PORT, 7);						//Server Port

	#if MX_WLAN_SMODE == 0								//Reset Security?
		Send_Command("GU0_0", 5);						//Security Off
	#endif

	return 0;

/*Macro_Initialise_End*/
}



char Send_Command(char * String, char MSZ_String)
{
/*Macro_Send_Command_Start*/

	char retval = 255;
	unsigned int timeout = 0;
	unsigned int attempts = 0;

	while ((retval) && (attempts < 300))					//Wait up to 1 minute - should be a few us for most commands.
	{
		if (cmd_state != 0)									//Ready to send command?
		{
			delay_ms(1);									//Small Delay
			cmd_state = 0;
		}

		FC_CAL_Bit_Low(MX_WLAN_TRIG_PORT, MX_WLAN_TRIG_PIN);		//Output 0 to get module into serial command mode

		delay_us(10);

		in_length = 0;										//Reset response length

		%a_UART_Send(0x3C);
		Send_String(String, MSZ_String, 0);					//Output command to WLAN module
		%a_UART_Send(0x3E);

		cmd_state = 1;
		timeout = 0;

		while ((cmd_state == 1) && (timeout < 5000))		//Wait up to .2 seconds for a reply
		{
			delay_us(40);
			timeout = timeout + 1;							//Wait here while we receive the reply
		}

		if (timeout == 5000)								//Timeout occurred - bail out
		{
			cmd_state = 0;
		}

		retval = Get_Reply();

		FC_CAL_Bit_High(MX_WLAN_TRIG_PORT, MX_WLAN_TRIG_PIN);		//Output 0 to get module into serial command mode

		attempts = attempts + 1;
	}

	return retval;

/*Macro_Send_Command_End*/
}


char Get_Reply()
{
/*Macro_Get_Reply_Start*/

	if (in_length > 0)
	{
		if (in_buffer[1] == 'S')
			return MX_WLAN_CMD_OK;
		else if (in_buffer[1] == 'F')
			return MX_WLAN_CMD_FAIL_ERR;
		else if (in_buffer[1] == '0')
			return MX_WLAN_CMD_STX_ERR;
		else if (in_buffer[1] == '1')
			return MX_WLAN_CMD_CMD_ERR;
		else if (in_buffer[1] == '2')
			return MX_WLAN_CMD_PARAM_ERR;
		else if (in_buffer[1] == '3')
			return MX_WLAN_CMD_ETX_ERR;
		else if (in_buffer[1] == 'E')
			return MX_WLAN_CMD_SERIAL;
	}
	else
		return MX_WLAN_CMD_TOUT;

/*Macro_Get_Reply_End*/
}



void Receive_Incoming (FCR_RETVAL, FCR_RETVAL_SIZE)
{
/*Macro_Receive_Incoming_Start*/

	char idx;

	for (idx=0; idx<in_length; idx++)
	{
		if (idx < FCR_RETVAL_SIZE)
			FCR_RETVAL[idx] = in_buffer[idx];
	}

	if (idx < FCR_RETVAL_SIZE)
		FCR_RETVAL[idx] = 0;

/*Macro_Receive_Incoming_End*/
}



void Send_String(char * String, char MSZ_String, char Append_CRLF)
{
/*Macro_Send_String_Start*/

	char idx;
	char varref;
	char num_as_string[3];

	for (idx = 0; String[idx] != 0; idx++)
	{
		if (String[idx] == 0)
		{
			break;
		}
		else if (String[idx] == '%')						//First % symbol
		{
			idx = idx + 1;									//Move to next character
			if (String[idx] == '%')
			{
				%a_UART_Send(String[idx]);		//Double % symbol = transmit the %
			}
			else
			{
				varref = String[idx] - '0';					//Calculate variable reference
				if (varref < NUM_VARS)
				{
					FCI_TOSTRING(Out_Variables[varref], num_as_string, 3);

					for (varref = 0; varref < 3; varref++)
					{
						if (num_as_string[varref] == 0)
						{
							break;
						}
						else
							%a_UART_Send(num_as_string[varref]);
					}
				}
			}
		}
		else
			%a_UART_Send(String[idx]);
	}

	if (Append_CRLF)
	{
		%a_UART_Send(0x0D);	//CR
		%a_UART_Send(0x0A);	//LF
	}

/*Macro_Send_String_End*/
}



char Check_For_Page_Requests (void)
/*Macro_Check_For_Page_Requests_Start*/

	char idx = 1;
	char length = 0;
	char page = 0;
	char varidx = 0;
	char value = 0;
	char finished = 0;
	char count[4] = {0,0,0,0};

	if((in_buffer[0] == '/') && (cmd_state == 0))			//Page Request has been received
	{
		if (in_buffer[1] == 0)		//If no page specified then default to page 1
		{
			page = 1;
		}
		else
		{
			while (in_buffer[idx] != 0 && in_buffer[idx] != '?')
			{
				#ifdef MX_WLAN_USE_PAGE1
					if (in_buffer[idx] == MX_WLAN_HTML_PAGE1[idx - 1])
					{
						count[0] = count[0] + 1;
					}
				#endif
				#ifdef MX_WLAN_USE_PAGE2
					if (in_buffer[idx] == MX_WLAN_HTML_PAGE2[idx - 1])
					{
						count[1] = count[1] + 1;
					}
				#endif
				#ifdef MX_WLAN_USE_PAGE3
					if (in_buffer[idx] == MX_WLAN_HTML_PAGE3[idx - 1])
					{
						count[2] = count[2] + 1;
					}
				#endif
				#ifdef MX_WLAN_USE_PAGE4
					if (in_buffer[idx] == MX_WLAN_HTML_PAGE4[idx - 1])
					{
						count[3] = count[3] + 1;
					}
				#endif

				idx = idx + 1;
			}
			length = idx - 1;			//Subtract slash and null termination

			#ifdef MX_WLAN_USE_PAGE1
				if (count[0] == length)
					page = 1;
			#endif
			#ifdef MX_WLAN_USE_PAGE2
				if (count[1] == length)
					page = 2;
			#endif
			#ifdef MX_WLAN_USE_PAGE3
				if (count[2] == length)
					page = 3;
			#endif
			#ifdef MX_WLAN_USE_PAGE4
				if (count[3] == length)
					page = 4;
			#endif

			if (in_buffer[idx] == '?')			//Pass Back Parameter
			{
				while (finished == 0)
				{
					idx = idx + 1;
					if (in_buffer[idx] >= '0' && in_buffer[idx] < '4')		// check for valid return variable
					{
						varidx = in_buffer[idx] - '0';
						idx = idx + 1;
						if (in_buffer[idx] == '=')							//Check that equals character is correct
						{
							idx = idx + 1;
							while (in_buffer[idx] != 0 && in_buffer[idx] != '&')
							{
								value = value * 10;
								value = value + in_buffer[idx] - '0';
								idx = idx + 1;
							}

							if (varidx < NUM_VARS)							//Check there is room in the array
								In_Variables[varidx] = value;

							if (in_buffer[idx] != '&')						//Check to see if there are more variables coming
								finished = 1;

							value = 0;						//Reset Count Value
						}
					}
				}
			}
		}

		Send_String("HTTP/1.0 200 OK", 15, 1);
		Send_String("Content-type: text/html", 23, 1);
		Send_String("", 0, 1);
		Send_String("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">", 62, 1);

		switch(page)
		{
			case 1:					//Page 1
				#ifdef MX_WLAN_USE_PAGE1
					SendHTMLPage1();
				#endif
				break;

			case 2:					//Page 2
				#ifdef MX_WLAN_USE_PAGE2
					SendHTMLPage2();
				#endif
				break;

			case 3:					//Page 3
				#ifdef MX_WLAN_USE_PAGE3
					SendHTMLPage3();
				#endif
				break;

			case 4:					//Page 4
				#ifdef MX_WLAN_USE_PAGE4
					SendHTMLPage4();
				#endif
				break;

			default:				//404 Not Found Page
				Send_String("404 - Page Not Found.", 21, 1);
				break;
		}
		in_buffer[0] = 0;			//Invalidate detection for next iteration
		return page;
	}
	return 0;

/*Macro_Check_For_Page_Requests_End*/
}


char Connect_To_SSID (char * SSID, char MSZ_SSID, char * Security_Key, char MSZ_Security_Key)
{
/*Macro_Connect_To_SSID_Start*/

	char retval = 255;

	#if MX_WLAN_MODEA == 3								//CLIENT MODE ONLY

		char outgoing[60];								//Does this value need to be bigger / smaller? 9 current max

		Send_Command("DI", 2);							//Scan for access points

		delay_s(2);

		FCI_CONCATENATE("GI", 2, SSID, MSZ_SSID, outgoing, 60);
		FCI_CONCATENATE(outgoing, 60, "_", 1, outgoing, 60);
		FCI_CONCATENATE(outgoing, 60, Security_Key, MSZ_Security_Key, outgoing, 60);

		retval = Send_Command(outgoing, 60);			//Attempt to connect to AP

	#endif

	return retval;

/*Macro_Connect_To_SSID_End*/
}


void Write_Swap_Variable(char index, char data)
{
/*Macro_Write_Swap_Variable_Start*/

	if (index >= NUM_VARS)
		return;

	Out_Variables[index] = data;

/*Macro_Write_Swap_Variable_End*/
}


char Read_Swap_Variable(char index)
{
/*Macro_Read_Swap_Variable_Start*/

	if (index >= NUM_VARS)
		return 0;

	return (In_Variables[index]);

/*Macro_Read_Swap_Variable_End*/
}


char Read_Connection_Status()
{
/*Macro_Read_Connection_Status_Start*/

	Send_Command("RC",2);
	return (in_buffer[2] - '0');			//In buffer should be "<S0>" not connected or "<S1>" connected

/*Macro_Read_Connection_Status_End*/
}

